import clsx from 'clsx';
import PropTypes from 'prop-types';
import DataTableHead from './TableHead';
import TablePagination from '../TablePagination';
import TableRow from '@material-ui/core/TableRow';
import MaterialTable from '@material-ui/core/Table';
import TableBody from '@material-ui/core/TableBody';
import TableCell from '@material-ui/core/TableCell';
import FuseScrollbars from '@fuse/core/FuseScrollbars';
import LinearProgress from '@material-ui/core/LinearProgress';

import Typography from "@material-ui/core/Typography";
import IconButton from "@material-ui/core/IconButton"; 
import { Print   ,PhotoLibrary, PictureAsPdf} from "@material-ui/icons";

const DataTable = ({
  page,
  count,
  hover,
  sortBy,
  columns,
  perPage,
  sortDir,
  rows = [],
  isLoading,
  handleSort,
  onRowClick,
  selectedRows,
  handlePageChange,
  isAllRowsSelected,
  handlePerPageChange,
  stickyHeader = false,
  isIndeterminateSelect,
  handleToggleSelectedRow,
  handleToggleAllSelectedRows,
  paginationClasses,
}) => {
  const handleChangePage = (event, newPage) => {
    handlePageChange(newPage + 1);
  };

  const handleChangeRowsPerPage = (event) => {
    handlePerPageChange(Number(event.target.value));
  };

  const renderCell = ({ row, col, isSelected }) => {
    if (typeof col.accessor === 'function') {
      return col.accessor({
        row,
        isSelected,
        selectedRows,
        handleToggleSelectedRow,
      });
    }
    return row[col.accessor];
  };

  return (
    <div className="w-full flex flex-col">
      <FuseScrollbars className="flex-grow overflow-x-auto">
        <MaterialTable
          role="table"
          className="min-w-xl"
          stickyHeader={stickyHeader}
          aria-labelledby="tableTitle"
        >
          <DataTableHead
            sortBy={sortBy}
            columns={columns}
            sortDir={sortDir}
            onSort={handleSort}
            selectedRows={selectedRows}
            isAllRowsSelected={isAllRowsSelected}
            isIndeterminateSelect={isIndeterminateSelect}
            onToggleAllSelectedRow={handleToggleAllSelectedRows}
          />

          <TableBody>
            <TableRow className={clsx('h-3', !isLoading && 'invisible')}>
              <TableCell colSpan={columns.length} padding="none">
                {isLoading && (
                  <LinearProgress
                    color="secondary"
                    classes={{
                      root: 'h-3',
                    }}
                  />
                )}
              </TableCell>
            </TableRow>
            {rows &&
              rows.map((row, index) => {
                const rowId = row.id || index;
                const isSelected = selectedRows[rowId] ? true : false;

                return (
                  <TableRow
                    role="row"
                    key={rowId}
                    hover={hover}
                    tabIndex={-1}
                    selected={isSelected}
                    className={clsx('h-44', hover && 'cursor-pointer')}
                    onClick={onRowClick ? () => onRowClick(row) : undefined}
                  >
                    {columns.map((col, i) => {
                      const id =
                        col.id ||
                        (typeof col.accessor === 'string'
                          ? col.accessor
                          : undefined);

                      return (
                        <TableCell
                          key={id}
                          role="cell"
                          scope="row"
                          align={col.align}
                          className={clsx('p-0')}
                          padding={col.disablePadding ? 'none' : 'default'}
                        >
                          {renderCell({
                            row: { ...row, id: rowId },
                            col,
                            isSelected,
                          })}
                        </TableCell>
                      );
                    })}
                  </TableRow>
                );
              })}
          </TableBody>
        </MaterialTable>
      </FuseScrollbars>
      <div className="flex-row flex justify-between bottom-0">
        <div className="px-8">
        
           تعداد سبد 
            
          <IconButton onClick={() => window.print()}>
            <Print />
          </IconButton>
          <IconButton onClick={() => window.print()}>
            <PictureAsPdf />
          </IconButton>
          <IconButton onClick={() => window.print()}>
            <PhotoLibrary />
          </IconButton>
        </div>

        {count ? (
          <TablePagination className=" justify-end"
            count={count}
            page={page - 1}
            rowsPerPage={perPage}
            classes={paginationClasses}
            onChangePage={handleChangePage}
            onChangeRowsPerPage={handleChangeRowsPerPage}
          />
        ) : null}
      </div>
    </div>
  );
};

DataTable.propTypes = {
  stickyHeader: PropTypes.bool,
  rows: PropTypes.array.isRequired,
  columns: PropTypes.array.isRequired,
  paginationClasses: PropTypes.object,
};

export default DataTable;

