import { useEffect, useReducer } from 'react';
import qs from 'qs';
import { useHistory, useLocation } from 'react-router-dom';

function useDataTable({
  data,
  columns,
  initialSortBy,
  initialSearch,
  initialSortDir,
  initialPage = 1,
  initialPerPage = 10,
  initialFilters = {},
  disableSortRemove = false,
}) {
  const initialState = {
    appliedFilters: {
      page: initialPage,
      perPage: initialPerPage,
      filters: initialFilters,
      search: initialSearch,
      sortBy: initialSortBy,
      sortDir: initialSortDir,
    },
    selectedRows: {},
    isAllRowsSelected: false,
    isIndeterminateSelect: false,
  };

  function reducer(state, action) {
    switch (action.type) {
      case 'SET_APPLIED_FILTERS':
        const appliedFilters = action.appliedFilters;

        if (!appliedFilters.page) {
          appliedFilters.page = 1;
        }

        if (!appliedFilters.perPage) {
          appliedFilters.perPage = initialPerPage;
        }

        appliedFilters.page = parseInt(appliedFilters.page);
        appliedFilters.perPage = parseInt(appliedFilters.perPage);

        return {
          ...state,
          appliedFilters,
          selectedRows: {},
          isAllRowsSelected: false,
          isIndeterminateSelect: false,
        };

      case 'TOGGLE_ROW_SELECTED':
        let isAllRowsSelected;
        const selectedRows = { ...state.selectedRows };

        if (action.isSelected) {
          selectedRows[action.id] = true;
          isAllRowsSelected = data.every((row) => selectedRows[row.id]);
        } else {
          isAllRowsSelected = false;
          delete selectedRows[action.id];
        }

        return {
          ...state,
          selectedRows,
          isAllRowsSelected,
          isIndeterminateSelect:
            !isAllRowsSelected && Object.keys(selectedRows).length !== 0,
        };

      case 'TOGGLE_ALL_ROW_SELECTED':
        const selecteditems = {};

        if (action.isChecked) {
          data.forEach((row) => {
            selecteditems[row.id] = true;
          });
        }

        return {
          ...state,
          selectedRows: selecteditems,
          isAllRowsSelected: action.isChecked,
          isIndeterminateSelect: false,
        };
      case 'SET_VALUE':
        return { ...state, [action.name]: action.value };
      default:
        return state;
    }
  }

  const [state, setState] = useReducer(reducer, initialState);
  const history = useHistory();
  const location = useLocation();

  const { search: locationSearch } = location;

  useEffect(() => {
    const appliedFilters = qs.parse(locationSearch, {
      ignoreQueryPrefix: true,
    });

    if (appliedFilters.perPage) {
      appliedFilters.perPage = parseInt(appliedFilters.perPage);
    }

    setState({
      type: 'SET_APPLIED_FILTERS',
      appliedFilters,
    });
  }, [locationSearch]);

  const handleToggleSelectedRow = (id, isSelected) => {
    setState({
      type: 'TOGGLE_ROW_SELECTED',
      id,
      isSelected,
    });
  };

  const handleToggleAllSelectedRows = (isChecked) => {
    setState({ type: 'TOGGLE_ALL_ROW_SELECTED', isChecked });
  };

  const handleAppliedFiltersChange = (filterName, value) => {
    let appliedFilters = {
      ...state.appliedFilters,
      [filterName]: value,
    };

    if (filterName !== 'page') {
      appliedFilters.page = 1;
    }

    if (filterName === 'sortBy') {
      appliedFilters.sortDir = 'asc';

      if (value === state.appliedFilters.sortBy) {
        switch (state.appliedFilters.sortDir) {
          case 'asc':
            appliedFilters.sortDir = 'desc';
            break;
          case 'desc':
            delete appliedFilters.sortBy;
            delete appliedFilters.sortDir;
            break;
          default:
            appliedFilters.sortDir = 'asc';
            break;
        }
      }
    }

    const location = createLocationFromAppliedFilters(appliedFilters);
    history.push(location);
  };

  const createLocationFromAppliedFilters = (appliedFilters) => {
    let urlQueryParams = {};
    Object.keys(appliedFilters).forEach((filterName) => {
      urlQueryParams[filterName] = appliedFilters[filterName];
    });

    if (parseInt(urlQueryParams.page) === 1) {
      urlQueryParams.page = null;
    }

    if (urlQueryParams.search === '') {
      urlQueryParams.search = null;
    }

    if (parseInt(urlQueryParams.perPage) === parseInt(initialPerPage)) {
      urlQueryParams.perPage = null;
    }

    return {
      pathname: location.pathname,
      search: qs.stringify(urlQueryParams, {
        encode: false,
        skipNulls: true,
        indices: false,
        arrayFormat: 'brackets',
      }),
      state: location.state,
    };
  };

  const handleSearch = (query) => {
    handleAppliedFiltersChange('search', query);
  };

  const handleClearSearch = (e) => {
    e.preventDefault();
    handleAppliedFiltersChange('search', '');
    setState({
      type: 'SET_VALUE',
      name: 'search',
      value: '',
    });
  };

  const handlePerPageChange = (value) => {
    // Todo Add perpage to states!!!! or maybe not?
    handleAppliedFiltersChange('perPage', value);
  };

  const handlePageChange = (pageNumber) => {
    handleAppliedFiltersChange('page', parseInt(pageNumber));
  };

  const handleSort = (id) => {
    handleAppliedFiltersChange('sortBy', id);
  };

  return {
    rows: data,
    columns,
    handleSort,
    handleSearch,
    handlePageChange,
    handleClearSearch,
    handlePerPageChange,
    handleAppliedFiltersChange,
    page: state.appliedFilters.page,
    sortBy: state.appliedFilters.sortBy,
    appliedFilters: state.appliedFilters,
    perPage: state.appliedFilters.perPage,
    filters: state.appliedFilters.filters,
    sortDir: state.appliedFilters.sortDir,
    search: state.appliedFilters.search,
    selectedRows: state.selectedRows,
    handleToggleSelectedRow,
    handleToggleAllSelectedRows,
    isAllRowsSelected: state.isAllRowsSelected,
    isIndeterminateSelect: state.isIndeterminateSelect,
  };
}

export default useDataTable;
