import {
  useRef,
  useMemo,
  useState,
  useEffect,
  forwardRef,
  useCallback,
} from 'react';
import {
  useTable,
  useSortBy,
  useRowSelect,
  usePagination,
  useGlobalFilter,
} from 'react-table';
import _ from '@lodash';
import PropTypes from 'prop-types';
import FuseUtils from '@fuse/utils';
import PropertyTable from './PropertyTable';
import Avatar from '@material-ui/core/Avatar';
import PropertySidebar from './PropertySidebar';
import { useFieldArray } from 'react-hook-form';
import Checkbox from '@material-ui/core/Checkbox';
import DeleteIcon from '@material-ui/icons/Delete';
import IconButton from '@material-ui/core/IconButton';
import Typography from '@material-ui/core/Typography';
import PropertyFormDialog from './PropertyFormDialog';
import useProducts from '../../hooks/useProducts';
import useCategories from '../../hooks/useCategories';
import useCategoryLabels from '../../hooks/useCategoryLabels';

const IndeterminateCheckbox = forwardRef(({ indeterminate, ...rest }, ref) => {
  const defaultRef = useRef();
  const resolvedRef = ref || defaultRef;

  useEffect(() => {
    resolvedRef.current.indeterminate = indeterminate;
  }, [resolvedRef, indeterminate]);

  return <Checkbox ref={resolvedRef} indeterminate={indeterminate} {...rest} />;
});

const PostProperty = ({ t, control, setValue, theme }) => {
  const [isDialogOpen, setIsDialogOpen] = useState(false);
  const [isEditDialog, setIsEditDialog] = useState(false);
  const [selectedProperty, setSelectedProperty] = useState(null);

  const { data: allCategoryLabels = [] } = useCategoryLabels();
  const { data: allCategories = [] } = useCategories();
  const { data: allProducts = [] } = useProducts();

  console.log({ allCategories, allCategoryLabels, allProducts });
  const { append, remove, fields: products } = useFieldArray({
    control,
    keyName: 'index',
    name: 'products',
  });

  const handleAddProperty = (data) => {
    append({ ...data, id: FuseUtils.generateGUID() });
  };

  const handleEditProperty = ({ index, ...data }) => {
    setValue(
      'products',
      products.map((item) => {
        const { index, ...properyObject } = item;
        return data.id === properyObject.id ? data : properyObject;
      }),
      { shouldDirty: true }
    );
  };

  const handleGroupDelete = useCallback(
    (ids) => {
      setValue(
        'products',
        products.filter((product) => !ids.includes(product.id)),
        { shouldDirty: true }
      );
    },
    [products, setValue]
  );

  const columns = useMemo(
    () => [
       
      {
        Header: t('NAME'),
        accessor: 'product_name',
        sortable: true,
      },
      {
        Header: t('PRICE'),
        accessor: 'product_after_discount',
        sortable: true,
      },
      {
        Header: t('DESCRIPTION'),
        accessor: 'product_info',
        Cell: ({ row }) => {
          const product_info = row.original.product_info.replace(
            /<(?:.|\n)*?>/gm,
            ' '
          );
          return (
            <Typography
              className="truncate"
              color="textSecondary"
              title={product_info}
            >
              {_.truncate(product_info, { length: 40 })}
            </Typography>
          );
        },
        sortable: true,
      },
      {
        id: 'action',
        width: 128,
        sortable: false,
        Cell: ({ row }) => (
          <div className="flex items-center">
            <IconButton
              onClick={(ev) => {
                ev.stopPropagation();
                remove(row.index);
              }}
            >
              <DeleteIcon />
            </IconButton>
          </div>
        ),
      },
    ],
    [handleGroupDelete, remove, t]
  );

  const handleFilterData = useCallback((rows, columnIds, globalFilterValue) => {
    if (!globalFilterValue) {
      return rows;
    }

    return rows.filter(
      (row) => row.original?.propertyLabel?.id === globalFilterValue
    );
  }, []);

  const tableInstance = useTable(
    {
      columns,
      data: products,
      autoResetPage: true,
      globalFilter: handleFilterData,
    },
    useGlobalFilter,
    useSortBy,
    usePagination,
    useRowSelect,
    (hooks) => {
      hooks.allColumns.push((_columns) => [
        // Let's make a column for selection
        {
          id: 'selection',
          sortable: false,
          Header: ({ getToggleAllRowsSelectedProps }) => (
            <div>
              <IndeterminateCheckbox {...getToggleAllRowsSelectedProps()} />
            </div>
          ),
          // The cell can use the individual row's getToggleRowSelectedProps method
          // to the render a checkbox
          Cell: ({ row }) => (
            <div>
              <IndeterminateCheckbox
                {...row.getToggleRowSelectedProps()}
                onClick={(ev) => ev.stopPropagation()}
              />
            </div>
          ),
        },
        ..._columns,
      ]);
    }
  );

  const {
    setGlobalFilter,
    state: { globalFilter = '' },
  } = tableInstance;

  const handleOpenAddDialog = () => {
    setIsDialogOpen(true);
    setIsEditDialog(false);
  };

  const handleOpenEditDialog = (row) => {
    setSelectedProperty(row.original);
    setIsEditDialog(true);
    setIsDialogOpen(true);
  };

  const handleCloseDialog = () => {
    setIsDialogOpen(false);
    setIsEditDialog(false);
    setSelectedProperty(null);
  };

  return (
    <div className="flex h-full">
      <PropertySidebar
        t={t}
        products={products}
        selectedLabel={globalFilter}
        allProperyLabels={allCategoryLabels}
        onOpenAddDialog={handleOpenAddDialog}
        onSelectedLabelChange={setGlobalFilter}
      />

      <PropertyTable
        t={t}
        data={products}
        tableInstance={tableInstance}
        onOpenEditDialog={handleOpenEditDialog}
      />

      {isDialogOpen && (
        <PropertyFormDialog
          theme={theme}
          edit={isEditDialog}
          isOpen={isDialogOpen}
          categories={allCategories}
          products={allProducts}
          onClose={handleCloseDialog}
          formState={selectedProperty}
          categoryLabels={allCategoryLabels}
          onSubmit={isEditDialog ? handleEditProperty : handleAddProperty}
        />
      )}
    </div>
  );
};

PostProperty.propTypes = {
  t: PropTypes.func.isRequired,
  theme: PropTypes.object.isRequired,
  setValue: PropTypes.func.isRequired,
  control: PropTypes.object.isRequired,
};

export default PostProperty;
