import { useState } from 'react';
import clsx from 'clsx';
import * as yup from 'yup';
import PropTypes from 'prop-types';
import Editor from 'app/components/Editor';
import AppBar from '@material-ui/core/AppBar';
import Button from '@material-ui/core/Button';
import Dialog from '@material-ui/core/Dialog';
import { useTranslation } from 'react-i18next';
import Toolbar from '@material-ui/core/Toolbar';
import TextField from '@material-ui/core/TextField';
import Typography from '@material-ui/core/Typography';
import { useForm, Controller } from 'react-hook-form';
import { yupResolver } from '@hookform/resolvers/yup';
import FuseChipSelect from '@fuse/core/FuseChipSelect';
import DialogActions from '@material-ui/core/DialogActions';
import DialogContent from '@material-ui/core/DialogContent';
import InputAdornment from '@material-ui/core/InputAdornment';
import DialogCloseButton from 'app/components/DialogCloseButton';

const validationSchema = yup.object().shape({
  productDetails: yup.object().required().nullable(),
  category: yup.object().required().nullable(),
  categoryLabel: yup.object().required().nullable(),
});

const defaultFormState = {
  
  product_name:'', 
  product_info: '',
  product_price: '0',
  product_discount: '0', 
  product_after_discount: '0',
  pid : 0,
  category: null,
  categoryLabel : null  ,
  productDetails: null,
};

function PropertyFormDialog({
  theme,
  isOpen,
  onClose,
  onSubmit,
  formState,
  edit = false,
  categories = [],
  products = [],
  categoryLabels = [],
}) {
  const [categoryOptions, setCategoryOptions] = useState([]);
  const [productOptions, setProductOptions] = useState([]);

  const {
    watch,
    reset,
    errors,
    control,
    register,
    setValue,
    getValues,
    handleSubmit,
  } = useForm({ 
    shouldUnregister: false,
    resolver: yupResolver(validationSchema),
    defaultValues: formState ? formState : defaultFormState,
  });

  const { t } = useTranslation('cms_translate');

  const handleSubmitForm = (data) => {
    onSubmit(data);
    if (!edit) {
      onClose();
    }
  };

  const handleCategoryLabelChange = (selectedCategoryLabel, onchange) => {
    onchange(selectedCategoryLabel);
    setValue('category', null);
    setValue('product', null);

    if (!selectedCategoryLabel || selectedCategoryLabel?.length === 0) {
      setCategoryOptions([]);
      setProductOptions([]);
    } else {
      setCategoryOptions(
        categories.filter((item) =>
          item.labels.includes(selectedCategoryLabel.id)
        )
      );
    }
  };

  const handleProductChange = (selectedProduct, onchange) => {
    onchange(selectedProduct);      
    
    setValue('product_name', selectedProduct.name);    
    setValue('product_info', selectedProduct.info);
    setValue('product_price', selectedProduct.priceTaxExcl);
    setValue('product_discount', selectedProduct.discount);
    setValue('product_after_discount', selectedProduct.priceTaxIncl);    
     setValue('pid', selectedProduct.id);    
  };

  const handleCategoryChange = (selectedCategory, onchange) => {
    onchange(selectedCategory);

    if (!selectedCategory || selectedCategory?.length === 0) {
      const { category, categoryLabel, ...defaultValues } = defaultFormState;
      reset({
        ...defaultValues,
        category: selectedCategory,
        categoryLabel: getValues('categoryLabel'),
      });
      setProductOptions([]);

      return;
    } else {
      setProductOptions(
        products.filter((item) =>
          item.categoriesObject?.some((c) => c.id === selectedCategory.id)
        )
      );
    }

   
  };

  const categoryLabel = watch('categoryLabel');
  const category = watch('category');

  return (
    <Dialog
      fullWidth
      open={isOpen}
      maxWidth="md"
      onClose={onClose}
      disableBackdropClick
      classes={{ paper: 'rounded-8' }}
    >
      <AppBar position="static" elevation={1}>
        <Toolbar className="flex w-full">
          <Typography variant="subtitle1" color="inherit">
            {edit ? t('EDIT_PROPERTY') : t('ADD_PROPERTY')}
          </Typography>
        </Toolbar>
        <DialogCloseButton onClick={onClose} />
      </AppBar>

      <form noValidate className="flex flex-col md:overflow-hidden">
        <DialogContent classes={{ root: 'p-28 mt-16' }}>
          <div className="-mx-3 flex flex-wrap mb-16">
            <Controller
              name="categoryLabel"
              control={control}
              render={(props) => (
                <FuseChipSelect
                  isClearable
                  value={props.value}
                  options={categoryLabels}
                  placeholder={t('SEARCH')}
                  error={!!errors?.categoryLabel}
                  getOptionValue={(option) => option.id}
                  getOptionLabel={(option) => option.title}
                  className="w-full md:w-1/3 px-3 mb-16 md:mb-0"
                  onChange={(value) =>
                    handleCategoryLabelChange(value, props.onChange)
                  }
                  textFieldProps={{
                    label: t('LABEL'),
                    variant: 'outlined',
                    InputLabelProps: { shrink: true },
                    helperText: errors?.categoryLabel?.message,
                  }}
                />
              )}
            />

            <Controller
              name="category"
              control={control}
              render={(props) => (
                <FuseChipSelect
                  isClearable
                  value={props.value}
                  options={categoryOptions}
                  error={!!errors?.category}
                  className="w-full md:w-1/3 px-3 mb-16 md:mb-0"
                  getOptionValue={(option) => option.id}
                  getOptionLabel={(option) => option.title}
                  placeholder={t('SEARCH')}
                  onChange={(value) =>
                    handleCategoryChange(value, props.onChange)
                  }
                  textFieldProps={{
                    label: t('LABEL'),
                    variant: 'outlined',
                    InputLabelProps: { shrink: true },
                    helperText: errors?.category?.message,
                  }}
                />
              )}
            />          
          </div>
       <div className="-mx-3 flex flex-wrap mb-16">
            <Controller
              name="productDetails"
              control={control}
              render={(props) => (
                <FuseChipSelect
                  isClearable
                  value={props.value}
                  options={productOptions}
                  error={!!errors?.productDetails}
                  className="w-full   px-3 mb-16 md:mb-0"
                  getOptionValue={(option) => option.id}
                  getOptionLabel={(option) => option.name}
                  placeholder={t('SEARCH')}
                  onChange={(value) =>
                    handleProductChange(value, props.onChange)
                  }
                  textFieldProps={{
                    label: t('PRODUCTS'),
                    variant: 'outlined',
                    InputLabelProps: { shrink: true },
                    helperText: errors?.productDetails?.message,
                  }}
                />
              )}
            />
          </div>

          <div>
            <TextField
              fullWidth
              type="text"
              name="product_name"
              className="mb-16"
              variant="outlined"
              inputRef={register} 
            />
          </div>
          <div className="-mx-3 flex flex-wrap mb-16">          
            <TextField
              fullWidth
              type="number"
              label={t('PRICE')}
              variant="outlined"
              inputRef={register}
              name="product_price"
              className="md:w-1/3 px-3 mb-16 md:mb-0"
              InputProps={{
                endAdornment: (
                  <InputAdornment position="start">{t('$')}</InputAdornment>
                ),
              }}
            />

            <TextField
              fullWidth
              type="number"
              variant="outlined"
              inputRef={register}
              label={t('PERCENT')}
              name="product_discount"
              className="md:w-1/3 px-3 mb-16 md:mb-0"
              InputProps={{
                endAdornment: (
                  <InputAdornment position="start">%</InputAdornment>
                ),
              }}
            />
              <TextField
              fullWidth
              type="number"
              label={t('product_after_discount')}
              variant="outlined"
              inputRef={register}
              name="product_after_discount"
              className="md:w-1/3 px-3 mb-16 md:mb-0"
              InputProps={{
                endAdornment: (
                  <InputAdornment position="start">{t('$')}</InputAdornment>
                ),
              }}
            />           
          </div>
    
          <Controller
            as={Editor}
            control={control}
            className="mb-16"
            name="product_info"
            direction={theme.direction}
          />
 
        </DialogContent>

        <DialogActions className="justify-between p-8">
          <div className="px-16">
            <Button
              type="button"
              color="primary"
              variant="contained"
              // disabled={!canBeSubmitted()}
              onClick={handleSubmit(handleSubmitForm)}
            >
              {t('SAVE')}
            </Button>
          </div>
        </DialogActions>
      </form>
    </Dialog>
  );
}

PropertyFormDialog.propTypes = {
  edit: PropTypes.bool,
  formState: PropTypes.object,
  isOpen: PropTypes.bool.isRequired,
  onClose: PropTypes.func.isRequired,
  theme: PropTypes.object.isRequired,
  onSubmit: PropTypes.func.isRequired,
  products: PropTypes.array.isRequired,
  categories: PropTypes.array.isRequired,
  categoryLabels: PropTypes.array.isRequired,
};

export default PropertyFormDialog;
