import { useState } from 'react';
import clsx from 'clsx';
import * as yup from 'yup';
import PropTypes from 'prop-types';
import Editor from 'app/components/Editor';
import AppBar from '@material-ui/core/AppBar';
import Button from '@material-ui/core/Button';
import Dialog from '@material-ui/core/Dialog';
import { useTranslation } from 'react-i18next';
import Toolbar from '@material-ui/core/Toolbar';
import TextField from '@material-ui/core/TextField';
import Typography from '@material-ui/core/Typography';
import { useForm, Controller } from 'react-hook-form';
import { yupResolver } from '@hookform/resolvers/yup';
import FuseChipSelect from '@fuse/core/FuseChipSelect';
import DialogActions from '@material-ui/core/DialogActions';
import DialogContent from '@material-ui/core/DialogContent';
import InputAdornment from '@material-ui/core/InputAdornment';
import DialogCloseButton from 'app/components/DialogCloseButton';

const validationSchema = yup.object().shape({
  property: yup.object().required().nullable(),
  propertyLabel: yup.object().required().nullable(),
});

const defaultFormState = {
  codeColor: '',
  voicelink: '',
  property: null,
  showType: 'both',
  PropertyPrice: 0,
  PropertyIcon: '',
  PropertyPercent: 0,
  propertyLabel: null,
  PropertyDescription: '',
};

function PropertyFormDialog({
  theme,
  isOpen,
  onClose,
  onSubmit,
  formState,
  edit = false,
  properties = [],
  properyLabels = [],
}) {
  const [propertyOptions, setPropertyOptions] = useState([]);
  const {
    watch,
    reset,
    errors,
    control,
    register,
    setValue,
    getValues,
    handleSubmit,
  } = useForm({
    shouldUnregister: false,
    resolver: yupResolver(validationSchema),
    defaultValues: formState ? formState : defaultFormState,
  });

  const { t } = useTranslation('cms_translate');

  const handleSubmitForm = (data) => {
    onSubmit(data);
    if (!edit) {
      onClose();
    }
  };

  const handlePropertyLabelChange = (selectedPropertyLabel, onchange) => {
    onchange(selectedPropertyLabel);
    setValue('property', null);

    if (!selectedPropertyLabel || selectedPropertyLabel?.length === 0) {
      setPropertyOptions([]);
    } else {
      setPropertyOptions(
        properties.filter((property) =>
          property.labels.includes(selectedPropertyLabel.id)
        )
      );
    }
  };

  const handlePropertyChange = (selectedProperty, onchange) => {
    onchange(selectedProperty);

    const propertyLabel = getValues('propertyLabel');

    if (!selectedProperty || selectedProperty?.length === 0) {
      const { property, properyLabel, ...defaultValues } = defaultFormState;
      reset({
        ...defaultValues,
        property: selectedProperty,
        properyLabel: propertyLabel,
      });
      return;
    }

    if (propertyLabel?.isColor && selectedProperty?.codeColor) {
      setValue('codeColor', selectedProperty.codeColor);
    }
    setValue('showType', selectedProperty.showType, { shouldDirty: true });
    setValue('PropertyPercent', selectedProperty.addPrice);
    setValue('PropertyPrice', selectedProperty.price);
    setValue('PropertyDescription', selectedProperty.description);
    setValue('voicelink', selectedProperty.voicelink);
    setValue('PropertyIcon', selectedProperty.iconSrc);
  };

  const propertyLabel = watch('propertyLabel');
  const property = watch('property');

  return (
    <Dialog
      fullWidth
      open={isOpen}
      maxWidth="md"
      onClose={onClose}
      disableBackdropClick
      classes={{ paper: 'rounded-8' }}
    >
      <AppBar position="static" elevation={1}>
        <Toolbar className="flex w-full">
          <Typography variant="subtitle1" color="inherit">
            {edit ? t('EDIT_PROPERTY') : t('ADD_PROPERTY')}
          </Typography>
        </Toolbar>
        <DialogCloseButton onClick={onClose} />
      </AppBar>

      <form noValidate className="flex flex-col md:overflow-hidden">
        <DialogContent classes={{ root: 'p-28 mt-16' }}>
          <div className="-mx-3 flex flex-wrap mb-16">
            <Controller
              name="propertyLabel"
              control={control}
              render={(props) => (
                <FuseChipSelect
                  isClearable
                  value={props.value}
                  options={properyLabels}
                  placeholder={t('SEARCH')}
                  error={!!errors?.propertyLabel}
                  getOptionValue={(option) => option.id}
                  getOptionLabel={(option) => option.title}
                  className="w-full md:w-1/2 px-3 mb-16 md:mb-0"
                  onChange={(value) =>
                    handlePropertyLabelChange(value, props.onChange)
                  }
                  textFieldProps={{
                    variant: 'outlined',
                    label: t('LABEL'),
                    InputLabelProps: { shrink: true },
                    helperText: errors?.propertyLabel?.message,
                  }}
                />
              )}
            />

            <Controller
              name="property"
              control={control}
              render={(props) => (
                <FuseChipSelect
                  isClearable
                  value={props.value}
                  options={propertyOptions}
                  error={!!errors?.property}
                  className="w-full md:w-1/2 px-3"
                  getOptionValue={(option) => option.id}
                  getOptionLabel={(option) => option.title}
                  placeholder={t('SEARCH')}
                  onChange={(value) =>
                    handlePropertyChange(value, props.onChange)
                  }
                  textFieldProps={{
                    label: t('LABEL'),
                    variant: 'outlined',
                    InputLabelProps: { shrink: true },
                    helperText: errors?.property?.message,
                  }}
                />
              )}
            />
          </div>

          <div className="-mx-3 flex flex-wrap mb-16">
            {/* <Controller
              name="PropertyPrice"
              control={control}
              render={(props) => (
                <TextField
                  value={props.value}
                  onChange={(value) =>
                    handlePropertyChange(value, props.onChange)
                  }
                  fullWidth
                  label={t('PRICE')}
                  variant="outlined"
                  className="md:w-1/3 px-3 mb-16 md:mb-0"
                  InputProps={{
                    endAdornment: (
                      <InputAdornment position="start">
                        {t('RIAL')}
                      </InputAdornment>
                    ),
                  }}
                />
              )}
            /> */}

            <TextField
              fullWidth
              type="number"
              label={t('PRICE')}
              variant="outlined"
              inputRef={register}
              name="PropertyPrice"
              className="md:w-1/3 px-3 mb-16 md:mb-0"
              InputProps={{
                endAdornment: (
                  <InputAdornment position="start">{t('RIAL')}</InputAdornment>
                ),
              }}
            />

            <TextField
              fullWidth
              type="number"
              variant="outlined"
              inputRef={register}
              label={t('PERCENT')}
              name="PropertyPercent"
              className="md:w-1/3 px-3 mb-16 md:mb-0"
              InputProps={{
                endAdornment: (
                  <InputAdornment position="start">%</InputAdornment>
                ),
              }}
            />

            <Controller
              name="showType"
              control={control}
              render={(props) => (
                <TextField
                  select
                  fullWidth
                  variant="outlined"
                  value={props.value}
                  label={t('SHOW_TYPE')}
                  SelectProps={{ native: true }}
                  className="md:w-1/3 px-3 mb-16 md:mb-0"
                  onChange={(e) => props.onChange(e.target.value)}
                >
                  <option value="Both">Both</option>
                  <option value="Top">Top</option>
                  <option value="Bottom">Bottom</option>
                </TextField>
              )}
            />
          </div>

          <div
            className={clsx((!property || !propertyLabel?.isColor) && 'hidden')}
          >
            <TextField
              fullWidth
              type="text"
              name="codeColor"
              className="mb-16"
              variant="outlined"
              inputRef={register}
              label={t('CODE_COLOR')}
            />
          </div>

          <Controller
            as={Editor}
            control={control}
            className="mb-16"
            name="PropertyDescription"
            direction={theme.direction}
          />

          <Controller
            fullWidth
            type="text"
            as={TextField}
            name="voicelink"
            control={control}
            className="mb-16"
            variant="outlined"
            label={t('VOICE_LINK')}
          />
        </DialogContent>

        <DialogActions className="justify-between p-8">
          <div className="px-16">
            <Button
              type="button"
              color="primary"
              variant="contained"
              // disabled={!canBeSubmitted()}
              onClick={handleSubmit(handleSubmitForm)}
            >
              {t('SAVE')}
            </Button>
          </div>
        </DialogActions>
      </form>
    </Dialog>
  );
}

PropertyFormDialog.propTypes = {
  edit: PropTypes.bool,
  formState: PropTypes.object,
  isOpen: PropTypes.bool.isRequired,
  onClose: PropTypes.func.isRequired,
  theme: PropTypes.object.isRequired,
  onSubmit: PropTypes.func.isRequired,
  properties: PropTypes.array.isRequired,
  properyLabels: PropTypes.array.isRequired,
};

export default PropertyFormDialog;
