import clsx from 'clsx';
import PropTypes from 'prop-types';
import Table from '@material-ui/core/Table';
import Paper from '@material-ui/core/Paper';
import Button from '@material-ui/core/Button';
import Toolbar from '@material-ui/core/Toolbar';
import Tooltip from '@material-ui/core/Tooltip';
import TableRow from '@material-ui/core/TableRow';
import { toPersianDigits } from 'app/utils/string';
import TableBody from '@material-ui/core/TableBody';
import TableCell from '@material-ui/core/TableCell';
import TableHead from '@material-ui/core/TableHead';
import { makeStyles } from '@material-ui/core/styles';
import IconButton from '@material-ui/core/IconButton';
import FilterListIcon from '@material-ui/icons/FilterList';
import TableContainer from '@material-ui/core/TableContainer';
import TableSortLabel from '@material-ui/core/TableSortLabel';
import TablePagination from '@material-ui/core/TablePagination';
import TablePaginationActions from 'app/components/TablePaginationActions';

const useStyles = makeStyles((theme) => ({
  paper: {
    width: '100%',
    marginBottom: theme.spacing(2),
  },
}));

const PropertyTable = ({ t, tableInstance, onOpenEditDialog, data }) => {
  const classes = useStyles();

  const {
    page,
    gotoPage,
    prepareRow,
    setPageSize,
    headerGroups,
    getTableProps,
    state: { pageIndex, pageSize },
  } = tableInstance;

  const handleChangePage = (event, newPage) => {
    gotoPage(newPage);
  };

  const handleChangeRowsPerPage = (event) => {
    setPageSize(Number(event.target.value));
  };

  return (
    <div className="w-full p-0 lg:p-4 lg:ltr:pl-24 lg:rtl:pr-24">
      <Paper className={classes.paper}>
        <Toolbar className="justify-between lg:hidden">
          <Button color="primary" onClick={() => {}} variant="contained">
            {t('ADD_PROPERTY')}
          </Button>
          <Tooltip title="Filter list">
            <IconButton aria-label="filter list">
              <FilterListIcon />
            </IconButton>
          </Tooltip>
        </Toolbar>

        <TableContainer>
          <Table {...getTableProps()}>
            <TableHead>
              {headerGroups.map((headerGroup) => (
                <TableRow {...headerGroup.getHeaderGroupProps()}>
                  {headerGroup.headers.map((column) => (
                    <TableCell
                      className="whitespace-no-wrap p-4 md:p-12"
                      {...(!column.sortable
                        ? column.getHeaderProps()
                        : column.getHeaderProps(column.getSortByToggleProps()))}
                      sortDirection={column.isSortedDesc ? 'desc' : 'asc'}
                      // padding="checkbox"
                    >
                      {column.sortable ? (
                        <TableSortLabel
                          active={column.isSorted}
                          direction={column.isSortedDesc ? 'desc' : 'asc'}
                        >
                          {column.render('Header')}
                        </TableSortLabel>
                      ) : (
                        column.render('Header')
                      )}
                    </TableCell>
                  ))}
                </TableRow>
              ))}
            </TableHead>

            <TableBody>
              {page.map((row, i) => {
                prepareRow(row);
                return (
                  <TableRow
                    {...row.getRowProps()}
                    hover
                    selected={row.isSelected}
                    className="truncate cursor-pointer"
                    onClick={() => onOpenEditDialog(row)}
                  >
                    {row.cells.map((cell) => {
                      return (
                        <TableCell
                          {...cell.getCellProps()}
                          className={clsx('p-4 md:p-12', cell.column.className)}
                        >
                          {cell.render('Cell')}
                        </TableCell>
                      );
                    })}
                  </TableRow>
                );
              })}
            </TableBody>
          </Table>
        </TableContainer>

        {data.length > 0 && (
          <TablePagination
            colSpan={10}
            component="div"
            page={pageIndex}
            count={data.length}
            rowsPerPage={pageSize}
            onChangePage={handleChangePage}
            labelRowsPerPage={t('row per page')}
            ActionsComponent={TablePaginationActions}
            classes={{ root: 'flex-shrink-0 border-0' }}
            onChangeRowsPerPage={handleChangeRowsPerPage}
            rowsPerPageOptions={[
              10,
              15,
              50,
              { label: 'All', value: data.length + 1 },
            ]}
            SelectProps={{
              native: false,
              inputProps: { 'aria-label': 'rows per page' },
            }}
            labelDisplayedRows={({ from, to, count }) => {
              return `${(from)} - ${(
                to
              )} from ${(count)}`;
            }}
          />
        )}
      </Paper>
    </div>
  );
};

PropertyTable.propTypes = {
  t: PropTypes.func.isRequired,
  data: PropTypes.array.isRequired,
  tableInstance: PropTypes.object.isRequired,
  onOpenEditDialog: PropTypes.func.isRequired,
};

export default PropertyTable;
