import _ from '@lodash';
import clsx from 'clsx';
import PropTypes from 'prop-types';
import List from '@material-ui/core/List';
import Icon from '@material-ui/core/Icon';
import CategoryChip from './CategoryChip';
import FuseAnimate from '@fuse/core/FuseAnimate';
import ListItem from '@material-ui/core/ListItem';
import { amber, red } from '@material-ui/core/colors';
import IconButton from '@material-ui/core/IconButton';
import { makeStyles } from '@material-ui/core/styles';
import Typography from '@material-ui/core/Typography';
import FuseAnimateGroup from '@fuse/core/FuseAnimateGroup';

const useStyles = makeStyles({
  item: {
    '&.completed': {
      background: 'rgba(0,0,0,0.03)',
      '& .category-title, & .category-notes': {
        textDecoration: 'line-through',
      },
    },
  },
});

function CategoryList({
  t,
  onUpdate,
  isLoading,
  categories,
  indexedLabels,
  onOpenEditDialog,
}) {
  const classes = useStyles();

  if (!isLoading && categories.length === 0) {
    return (
      <FuseAnimate delay={100}>
        <div className="flex flex-1 items-center justify-center h-full">
          <Typography color="textSecondary" variant="h5">
            {t('THERE_ARE_NO_CATEGORIES')}
          </Typography>
        </div>
      </FuseAnimate>
    );
  }

  return (
    <List className="p-0">
      <FuseAnimateGroup
        enter={{
          animation: 'transition.slideUpBigIn',
        }}
      >
        {categories.map((category) => (
          <ListItem
            dense
            button
            key={category.id}
            className={clsx(
              classes.item,
              { completed: category.completed },
              'border-solid border-b-1 py-16 px-0 sm:px-8'
            )}
            onClick={(ev) => {
              ev.preventDefault();
              onOpenEditDialog(category);
            }}
          >
            <div className="px-8"> 
             { category.imageSrc ? (
                 <img  className="w-32 sm:w-48 rounded"  src={category.imageSrc}  />
                 ) : (    <img  className="w-32 sm:w-48 rounded"
                      src={`${process.env.PUBLIC_URL}/assets/images/logos/logo.png`}  />  )
             }         
            </div> 
            <div className="flex flex-1 flex-col relative overflow-hidden px-8">
              <Typography
                variant="subtitle1"
                className="category-title truncate"
                color={category.completed ? 'textSecondary' : 'inherit'}
              >
                {category.title}
              </Typography>

              <Typography
                color="textSecondary"
                className="category-notes truncate"
              >
                {_.truncate(category.notes.replace(/<(?:.|\n)*?>/gm, ''), {
                  length: 180,
                })}
              </Typography>

              <div className={clsx(classes.labels, 'flex -mx-2')}>
                {category.labels.map((labelId) => {
                  const label = indexedLabels[labelId];
                  return label ? (
                    <CategoryChip
                      key={labelId}
                      title={label.title}
                      className="mx-2 mt-4"
                      color={label.color}
                    />
                  ) : null;
                })}
              </div>
            </div>

            <div className="px-8">
              <IconButton
                onClick={(ev) => {
                  ev.preventDefault();
                  ev.stopPropagation();
                  onUpdate({
                    ...category,
                    important: !category.important,
                  });
                }}
              >
                {category.important ? (
                  <Icon style={{ color: red[500] }}>error</Icon>
                ) : (
                  <Icon>error_outline</Icon>
                )}
              </IconButton>
              <IconButton
                onClick={(ev) => {
                  ev.preventDefault();
                  ev.stopPropagation();
                  onUpdate({
                    ...category,
                    starred: !category.starred,
                  });
                }}
              >
                {category.starred ? (
                  <Icon style={{ color: amber[500] }}>star</Icon>
                ) : (
                  <Icon>star_outline</Icon>
                )}
              </IconButton>
            </div>
          </ListItem>
        ))}
      </FuseAnimateGroup>
    </List>
  );
}

CategoryList.propTypes = {
  t: PropTypes.func.isRequired,
  onUpdate: PropTypes.func.isRequired,
  isLoading: PropTypes.bool.isRequired,
  categories: PropTypes.array.isRequired,
  indexedLabels: PropTypes.object.isRequired,
  onOpenEditDialog: PropTypes.func.isRequired,
};

export default CategoryList;
