import { useEffect, useReducer, useRef } from 'react';
import _ from '@lodash';
import FuseUtils from '@fuse/utils';
import PropertyList from './PropertyList';
import { useParams } from 'react-router-dom';
import Spinner from 'app/components/Spinner';
import { useTranslation } from 'react-i18next';
import PropertiesHeader from './PropertiesHeader';
import useProperties from '../hooks/useProperties';
import PropertiesToolbar from './PropertiesToolbar';
import PropertyFormDialog from './PropertyFormDialog';
import FusePageCarded from '@fuse/core/FusePageCarded';
import useSaveProperty from '../hooks/useSaveProperty';
import usePropertyLabels from '../hooks/usePropertyLabels';
import PropertiesSidebarHeader from './PropertiesSidebarHeader';
import PropertiesSidebarContent from './PropertiesSidebarContent';

function getFilteredArray(entities, _searchText) {
  if (_searchText.length === 0) {
    return entities;
  }
  return FuseUtils.filterArrayByString(entities, _searchText);
}

function reducer(state, action) {
  switch (action.type) {
    case 'SET_FILTERED_DATA':
      const { properties, searchText, orderBy, orderDescending } = action;
      let filteredData = [];
      if (properties) {
        filteredData = _.orderBy(
          getFilteredArray(properties, searchText),
          [orderBy],
          [orderDescending ? 'desc' : 'asc']
        );
      }
      return { ...state, filteredData };

    case 'SET_INDEXED_LABELS':
      return { ...state, indexedLabels: action.indexedLabels };

    case 'TOGGLE_ORDER_DESCENDING':
      return { ...state, orderDescending: !state.orderDescending };

    case 'SET_SEARCH_TEXT':
      return { ...state, searchText: action.searchText };

    case 'SET_ORDER_BY':
      return { ...state, orderBy: action.orderBy };

    case 'OPEN_ADD_DIALOG':
      return { ...state, isDialogOpen: true, isEditDialog: false };

    case 'OPEN_EDIT_DIALOG':
      return {
        ...state,
        isDialogOpen: true,
        isEditDialog: true,
        selectedProperty: action.selectedProperty,
      };

    case 'CLOSE_DIALOG':
      return {
        ...state,
        isDialogOpen: false,
        isEditDialog: false,
        selectedProperty: null,
      };

    default:
      throw new Error();
  }
}

const initialState = {
  orderBy: '',
  searchText: '',
  indexedLabels: {},
  filteredData: null,
  isDialogOpen: false,
  isEditDialog: false,
  selectedProperty: null,
  orderDescending: false,
};

function Properties() {
  const [state, setState] = useReducer(reducer, initialState);

  const {
    orderBy,
    searchText,
    filteredData,
    isDialogOpen,
    isEditDialog,
    indexedLabels,
    orderDescending,
    selectedProperty,
  } = state;

  const pageLayout = useRef(null);
  const { labelHandle, filterHandle, folderHandle } = useParams();

  const { t } = useTranslation('properties');

  const { data: labels = [], isLoading: isLabelsLoading } = usePropertyLabels();
  const { data: properties, isLoading } = useProperties({
    labelHandle,
    filterHandle,
    folderHandle,
  });

  const [saveProperty] = useSaveProperty();

  useEffect(() => {
    console.log('useEffect', { searchText, orderBy, orderDescending });

    setState({
      type: 'SET_FILTERED_DATA',
      orderBy,
      properties,
      searchText,
      orderDescending,
    });
  }, [orderBy, orderDescending, properties, searchText]);

  useEffect(() => {
    if (properties?.length > 0 && labels.length > 0) {
      console.log('indexedLabels useEffect');

      const indexedLabels = {};
      labels.forEach((label) => {
        indexedLabels[label.id] = label;
      });
      setState({ type: 'SET_INDEXED_LABELS', indexedLabels });
    }
  }, [properties?.length, labels]);

  const handleUpdateProperty = (property) => {
    return saveProperty({
      property,
      labelHandle,
      filterHandle,
      folderHandle,
      isEdit: true,
    });
  };

  const handleToggleOrderDescending = () => {
    setState({ type: 'TOGGLE_ORDER_DESCENDING' });
  };

  const handleSetSearchText = (searchText) => {
    setState({ type: 'SET_SEARCH_TEXT', searchText });
  };

  const handleSetOrderBy = (orderBy) => {
    setState({ type: 'SET_ORDER_BY', orderBy });
  };

  const handleOpenAddDialog = () => {
    setState({ type: 'OPEN_ADD_DIALOG' });
  };

  const handleOpenEditDialog = (selectedProperty) => {
    setState({ type: 'OPEN_EDIT_DIALOG', selectedProperty });
  };

  const handleCloseDialog = () => {
    setState({ type: 'CLOSE_DIALOG' });
  };

  const handleSubmit = (property) => {
    return saveProperty({
      property,
      labelHandle,
      filterHandle,
      folderHandle,
      isEdit: isEditDialog,
    });
  };

  if (!filteredData) {
    return null;
  }

  return (
    <>
      {isLoading && <Spinner />}
      <FusePageCarded
        classes={{
          root: 'w-full',
          header: 'items-center min-h-72 h-72 sm:h-136 sm:min-h-136',
        }}
        header={
          <PropertiesHeader
            t={t}
            searchText={searchText}
            pageLayout={pageLayout}
            onOpenAddDialog={handleOpenAddDialog}
            onSearchTextChange={handleSetSearchText}
          />
        }
        contentToolbar={
          <PropertiesToolbar
            t={t}
            orderBy={orderBy}
            orderDescending={orderDescending}
            onOrderByChange={handleSetOrderBy}
            onToggleOrderDescending={handleToggleOrderDescending}
          />
        }
        content={
          <PropertyList
            t={t}
            isLoading={isLoading}
            properties={filteredData}
            indexedLabels={indexedLabels}
            onUpdate={handleUpdateProperty}
            onOpenEditDialog={handleOpenEditDialog}
          />
        }
        leftSidebarHeader={<PropertiesSidebarHeader t={t} />}
        leftSidebarContent={
          <PropertiesSidebarContent
            t={t}
            labels={labels}
            isLabelsLoading={isLabelsLoading}
          />
        }
        ref={pageLayout}
        innerScroll
      />

      {isDialogOpen && (
        <PropertyFormDialog
          t={t}
          labels={labels}
          edit={isEditDialog}
          onSubmit={handleSubmit}
          data={selectedProperty}
          onClose={handleCloseDialog}
          indexedLabels={indexedLabels}
        />
      )}
    </>
  );
}

export default Properties;
