import * as yup from 'yup';
import { useState } from 'react';
import PropTypes from 'prop-types';
import { useParams } from 'react-router';
import Chip from '@material-ui/core/Chip';
import Icon from '@material-ui/core/Icon';
import Menu from '@material-ui/core/Menu';
import Editor from 'app/components/Editor';
import { getImageUrl } from 'app/utils/url';
import AppBar from '@material-ui/core/AppBar';
import Avatar from '@material-ui/core/Avatar';
import Dialog from '@material-ui/core/Dialog';
import ImageBox from 'app/components/ImageBox';
import Divider from '@material-ui/core/Divider';
import Toolbar from '@material-ui/core/Toolbar';
import MenuItem from '@material-ui/core/MenuItem';
import TextField from '@material-ui/core/TextField';
import { useTheme } from '@material-ui/core/styles';
import { amber, red } from '@material-ui/core/colors';
import IconButton from '@material-ui/core/IconButton';
import Typography from '@material-ui/core/Typography';
import { useForm, Controller } from 'react-hook-form';
import { yupResolver } from '@hookform/resolvers/yup';
import { makeStyles } from '@material-ui/core/styles';
import FuseChipSelect from '@fuse/core/FuseChipSelect';
import LoadingButton from 'app/components/LoadingButton';
import ListItemIcon from '@material-ui/core/ListItemIcon';
import ListItemText from '@material-ui/core/ListItemText';
import useDeleteProperty from '../hooks/useDeleteProperty';
import DialogActions from '@material-ui/core/DialogActions';
import DialogContent from '@material-ui/core/DialogContent';
import InputAdornment from '@material-ui/core/InputAdornment';
import DialogCloseButton from 'app/components/DialogCloseButton';

const useStyles = makeStyles((theme) => ({
  dialogContent: {
    padding: '0 !important',
  },
}));

const validationSchema = yup.object().shape({
  title: yup.string().required(),
  price: yup.string().required(),
  addPrice: yup.string().required(),
});

const defaultFormState = {
  title: '',
  price: '',
  labels: [],
  iconSrc: '',
  addPrice: '',
  showType: '',
  codeColor: '',
  voicelink: '',
  starred: false,
  description: '',
  important: false,
};

const types = [
  { id: 'Both', title: 'Both' },
  { id: 'Top', title: 'Top' },
  { id: 'Bottom', title: 'Bottom' },
];

function PropertyFormDialog({
  t,
  edit,
  data,
  labels,
  onClose,
  onSubmit,
  indexedLabels,
}) {
  const [labelMenuEl, setLabelMenuEl] = useState(null);

  const classes = useStyles();
  const theme = useTheme();

  const {
    watch,
    reset,
    errors,
    control,
    register,
    setValue,
    getValues,
    handleSubmit,
    formState: { isSubmitting },
  } = useForm({
    shouldUnregister: false,
    resolver: yupResolver(validationSchema),
    defaultValues: edit ? data : defaultFormState,
  });

  const [deleteProperty, { isLoading: isDeleting }] = useDeleteProperty();

  const urlParams = useParams();

  const handleSubmitForm = (data) => {
    return onSubmit(data).then((response) => {
      if (edit) {
        reset(response);
      } else {
        onClose();
      }
    });
  };

  function handleLabelMenuOpen(event) {
    setLabelMenuEl(event.currentTarget);
  }

  function handleLabelMenuClose(event) {
    setLabelMenuEl(null);
  }

  const handleToggleImportant = () => {
    const isImportant = getValues('important');
    setValue('important', !isImportant);
  };

  const handleToggleStarred = () => {
    const isStarred = getValues('starred');
    setValue('starred', !isStarred);
  };

  function handleToggleLabel(event, selectedLabel, isChecked) {
    event.stopPropagation();
    const labels = getValues('labels');

    setValue(
      'labels',
      isChecked
        ? [...labels, selectedLabel.id]
        : labels.filter((labelId) => labelId !== selectedLabel.id),
      { shouldDirty: true }
    );
  }

  const handleUploadChange = (e, fieldName) => {
    const file = e.target.files[0];

    setValue(fieldName + 'Src', URL.createObjectURL(file));
    setValue(fieldName + 'File', file);

    e.target.value = null;
  };

  const handleDeleteImage = (fieldName) => {
    setValue(fieldName + 'Src', '');
    setValue(fieldName + 'File', '');
  };

  const handleDeleteProperty = () => {
    deleteProperty({ id: data?.id, ...urlParams }).then(() => {
      onClose();
    });
  };

  const formValues = watch([
    'labels',
    'starred',
    'iconSrc',
    'iconFile',
    'important',
  ]);

  return (
    <Dialog
      fullWidth
      open={true}
      maxWidth="sm"
      onClose={onClose}
      disableBackdropClick
      classes={{ paper: 'rounded-8' }}
    >
      <AppBar position="static" elevation={1}>
        <Toolbar className="flex w-full">
          <Typography variant="subtitle1" color="inherit">
            {edit ? t('EDIT_PROPERTY') : t('ADD_NEW_PROPERTY')}
          </Typography>
        </Toolbar>
        <DialogCloseButton onClick={onClose} />
      </AppBar>

      <form
        noValidate
        onSubmit={handleSubmit(handleSubmitForm)}
        className="flex flex-col md:overflow-hidden"
      >
        <DialogContent classes={{ root: classes.dialogContent }}>
          <div className="mb-16">
            <div className="flex items-center p-12">
              <div
                className="flex items-center justify-start"
                aria-label="Toggle star"
              >
                <IconButton onClick={handleToggleImportant}>
                  {formValues.important ? (
                    <Icon style={{ color: red[500] }}>error</Icon>
                  ) : (
                    <Icon>error_outline</Icon>
                  )}
                </IconButton>

                <IconButton onClick={handleToggleStarred}>
                  {formValues.starred ? (
                    <Icon style={{ color: amber[500] }}>star</Icon>
                  ) : (
                    <Icon>star_outline</Icon>
                  )}
                </IconButton>
                <div>
                  <IconButton
                    aria-owns={labelMenuEl ? 'label-menu' : null}
                    aria-haspopup="true"
                    onClick={handleLabelMenuOpen}
                  >
                    <Icon>label</Icon>
                  </IconButton>
                  <Menu
                    id="label-menu"
                    anchorEl={labelMenuEl}
                    open={Boolean(labelMenuEl)}
                    onClose={handleLabelMenuClose}
                  >
                    {labels.length > 0 &&
                      labels.map((label) => {
                        const isChecked = formValues.labels.includes(label.id);
                        return (
                          <MenuItem
                            onClick={(ev) =>
                              handleToggleLabel(ev, label, !isChecked)
                            }
                            key={label.id}
                          >
                            <ListItemIcon className="min-w-24">
                              <Icon color="action">
                                {isChecked
                                  ? 'check_box'
                                  : 'check_box_outline_blank'}
                              </Icon>
                            </ListItemIcon>
                            <ListItemText
                              className="mx-8"
                              primary={label.title}
                              disableTypography
                            />
                            <ListItemIcon className="min-w-24">
                              <Icon
                                style={{ color: label.color }}
                                color="action"
                              >
                                label
                              </Icon>
                            </ListItemIcon>
                          </MenuItem>
                        );
                      })}
                  </Menu>
                </div>
              </div>
            </div>
            <Divider className="mx-24" />
          </div>

          {formValues.labels?.length > 0 && (
            <div className="flex flex-wrap w-full px-12 sm:px-20 mb-16">
              {formValues.labels.map((labelId) => {
                const label = indexedLabels[labelId];
                return label ? (
                  <Chip
                    key={labelId}
                    label={label.title}
                    className="mx-4 my-4"
                    classes={{ label: 'px-8' }}
                    onDelete={(ev) => handleToggleLabel(ev, label, false)}
                    avatar={
                      <Avatar classes={{ colorDefault: 'bg-transparent' }}>
                        <Icon
                          className="text-20"
                          style={{ color: label.color }}
                        >
                          label
                        </Icon>
                      </Avatar>
                    }
                  />
                ) : null;
              })}
            </div>
          )}

          <div className="px-16 sm:px-24">
            <TextField
              required
              fullWidth
              autoFocus
              type="text"
              name="title"
              className="mb-16"
              label={t('TITLE')}
              variant="outlined"
              inputRef={register}
              error={!!errors?.title}
              helperText={errors?.title?.message}
            />
            <Controller
              isClearable
              name="showType"
              options={types}
              control={control}
              as={FuseChipSelect}
              isSearchable={false}
              placeholder="showType"
              className="mt-8 mb-24"
              getOptionValue={(option) => option.id}
              getOptionLabel={(option) => option.title}
              textFieldProps={{
                variant: 'outlined',
                label: 'showType',
                InputLabelProps: { shrink: true },
              }}
            />

            <div className="-mx-3 flex flex-wrap mb-16">
              <TextField
                required
                fullWidth
                name="price"
                type="number"
                label={t('PRICE')}
                variant="outlined"
                inputRef={register}
                error={!!errors?.price}
                helperText={errors?.price?.message}
                className="md:w-1/2 px-3 mb-16 md:mb-0"
                InputProps={{
                  endAdornment: (
                    <InputAdornment position="start">
                      {t('RIAL')}
                    </InputAdornment>
                  ),
                }}
              />

              <TextField
                required
                fullWidth
                type="number"
                name="addPrice"
                variant="outlined"
                inputRef={register}
                label={t('PERCENT')}
                error={!!errors?.addPrice}
                helperText={errors?.addPrice?.message}
                className="md:w-1/2 px-3 mb-16 md:mb-0"
                InputProps={{
                  endAdornment: (
                    <InputAdornment position="start">%</InputAdornment>
                  ),
                }}
              />
            </div>

            <TextField
              fullWidth
              type="text"
              name="voicelink"
              className="mb-16"
              variant="outlined"
              inputRef={register}
              label={t('VOICE_LINK')}
            />

            <TextField
              fullWidth
              type="text"
              name="codeColor"
              className="mb-16"
              variant="outlined"
              inputRef={register}
              label={t('CODE_COLOR')}
            />

            <Controller
              as={Editor}
              control={control}
              className="mb-16"
              name="description"
              direction={theme.direction}
            />

            <div className="flex items-center flex-wrap -mx-16 mb-32">
              <ImageBox
                t={t}
                name="icon"
                accept="image/png"
                onDelete={() => handleDeleteImage('icon')}
                onChange={(e) => handleUploadChange(e, 'icon')}
                imageSource={
                  formValues.iconFile
                    ? formValues.iconSrc
                    : getImageUrl(formValues.iconSrc)
                }
              />
            </div>
          </div>
        </DialogContent>

        <DialogActions className="justify-between p-8">
          <div className="px-16">
            <LoadingButton
              type="submit"
              color="primary"
              variant="contained"
              isLoading={isSubmitting || isDeleting}
              className="whitespace-no-wrap normal-case"
            >
              {t('SAVE')}
            </LoadingButton>
          </div>
          {edit && (
            <IconButton
              disabled={isDeleting}
              className="min-w-auto"
              onClick={handleDeleteProperty}
            >
              <Icon>delete</Icon>
            </IconButton>
          )}
        </DialogActions>
      </form>
    </Dialog>
  );
}

PropertyFormDialog.propTypes = {
  data: PropTypes.object,
  t: PropTypes.func.isRequired,
  edit: PropTypes.bool.isRequired,
  labels: PropTypes.array.isRequired,
  onClose: PropTypes.func.isRequired,
  onSubmit: PropTypes.func.isRequired,
  indexedLabels: PropTypes.object.isRequired,
};

export default PropertyFormDialog;
