import { useEffect, useRef, useState } from 'react';
import _ from '@lodash';
import FuseUtils from '@fuse/utils';
import { useParams } from 'react-router-dom';
import Spinner from 'app/components/Spinner';
import { useTranslation } from 'react-i18next';
import PropertyLabelList from './PropertyLabelList';
import FusePageCarded from '@fuse/core/FusePageCarded';
import PropertyLabelsHeader from './PropertyLabelsHeader';
import usePropertyLabels from '../hooks/usePropertyLabels';
import PropertyLabelsToolbar from './PropertyLabelsToolbar';
import PropertyLabelFormDialog from './PropertyLabelFormDialog';
import useSavePropertyLabel from '../hooks/useSavePropertyLabel';
import PropertyLabelsSidebarHeader from './PropertyLabelsSidebarHeader';
import PropertyLabelsSidebarContent from './PropertyLabelsSidebarContent';

function PropertyLabels() {
  const [isDialogOpen, setIsDialogOpen] = useState(false);
  const [isEditDialog, setIsEditDialog] = useState(false);
  const [selectedLabel, setSelectedLabel] = useState(null);

  const [filteredData, setFilteredData] = useState(null);
  const [searchText, setSearchText] = useState('');
  const [orderBy, setOrderBy] = useState('');
  const [orderDescending, setOrderDescending] = useState(false);

  const pageLayout = useRef(null);

  const { filterHandle } = useParams();

  const { t } = useTranslation('properties');

  const { data: labels, isLoading } = usePropertyLabels();

  const [savePropertyLabel] = useSavePropertyLabel();

  useEffect(() => {
    console.log('useEffect', {
      searchText,
      orderBy,
      orderDescending,
      filterHandle,
    });
    function getFilteredArray(entities, _searchText) {
      if (_searchText.length === 0) {
        return labels;
      }
      return FuseUtils.filterArrayByString(labels, _searchText);
    }

    if (labels) {
      let filterdData = _.orderBy(
        getFilteredArray(labels, searchText),
        [orderBy],
        [orderDescending ? 'desc' : 'asc']
      );

      if (filterHandle && filterHandle !== 'all') {
        filterdData = filterdData.filter((item) => item[filterHandle]);
      }

      setFilteredData(filterdData);
    } else {
      setFilteredData([]);
    }
  }, [labels, orderBy, orderDescending, searchText, filterHandle]);

  const handleUpdateLabel = (propertyLabel) => {
    return savePropertyLabel({
      propertyLabel,
      isEdit: true,
    });
  };

  const handleToggleOrderDescending = () => {
    setOrderDescending((value) => !value);
  };

  const handleOpenAddDialog = () => {
    setIsDialogOpen(true);
    setIsEditDialog(false);
  };

  const handleOpenEditDialog = (selectedLabel) => {
    setSelectedLabel(selectedLabel);
    setIsEditDialog(true);
    setIsDialogOpen(true);
  };

  const handleCloseDialog = () => {
    setSelectedLabel(null);
    setIsDialogOpen(false);
    setIsEditDialog(false);
  };

  const handleSubmit = (propertyLabel) => {
    return savePropertyLabel({ propertyLabel, isEdit: isEditDialog });
  };

  if (!filteredData) {
    return null;
  }

  return (
    <>
      {isLoading && <Spinner />}
      <FusePageCarded
        classes={{
          root: 'w-full',
          header: 'items-center min-h-72 h-72 sm:h-136 sm:min-h-136',
        }}
        header={
          <PropertyLabelsHeader
            t={t}
            searchText={searchText}
            pageLayout={pageLayout}
            onSearchTextChange={setSearchText}
          />
        }
        contentToolbar={
          <PropertyLabelsToolbar
            t={t}
            orderBy={orderBy}
            onOrderByChange={setOrderBy}
            orderDescending={orderDescending}
            onToggleOrderDescending={handleToggleOrderDescending}
          />
        }
        content={
          <PropertyLabelList
            t={t}
            isLoading={isLoading}
            labels={filteredData}
            onUpdate={handleUpdateLabel}
            onOpenEditDialog={handleOpenEditDialog}
          />
        }
        leftSidebarHeader={<PropertyLabelsSidebarHeader t={t} />}
        leftSidebarContent={
          <PropertyLabelsSidebarContent
            t={t}
            onOpenAddDialog={handleOpenAddDialog}
          />
        }
        ref={pageLayout}
        innerScroll
      />

      {isDialogOpen && (
        <PropertyLabelFormDialog
          t={t}
          edit={isEditDialog}
          data={selectedLabel}
          onSubmit={handleSubmit}
          onClose={handleCloseDialog}
        />
      )}
    </>
  );
}

export default PropertyLabels;
