import { useEffect, useReducer, useRef } from 'react';
import _ from '@lodash';
import FuseUtils from '@fuse/utils';
import CategoryList from './CategoryList';
import { useParams } from 'react-router-dom';
import Spinner from 'app/components/Spinner';
import { useTranslation } from 'react-i18next';
import StatusHeader from './StatusHeader';
import useStatus from '../hooks/useStatus';
import StatusToolbar from './StatusToolbar';
import CategoryFormDialog from './CategoryFormDialog';
import FusePageCarded from '@fuse/core/FusePageCarded';
import useSaveCategory from '../hooks/useSaveCategory';
import useCategoryLabels from '../hooks/useCategoryLabels';
import StatusSidebarHeader from './StatusSidebarHeader';
import StatusSidebarContent from './StatusSidebarContent';

function getFilteredArray(entities, _searchText) {
  if (_searchText.length === 0) {
    return entities;
  }
  return FuseUtils.filterArrayByString(entities, _searchText);
}

function reducer(state, action) {
  switch (action.type) {
    case 'SET_FILTERED_DATA':
      const { status, searchText, orderBy, orderDescending } = action;
      let filteredData = [];
      if (status) {
        filteredData = _.orderBy(
          getFilteredArray(status, searchText),
          [orderBy],
          [orderDescending ? 'desc' : 'asc']
        );
      }
      return { ...state, filteredData };

    case 'SET_INDEXED_LABELS':
      return { ...state, indexedLabels: action.indexedLabels };

    case 'TOGGLE_ORDER_DESCENDING':
      return { ...state, orderDescending: !state.orderDescending };

    case 'SET_SEARCH_TEXT':
      return { ...state, searchText: action.searchText };

    case 'SET_ORDER_BY':
      return { ...state, orderBy: action.orderBy };

    case 'OPEN_ADD_DIALOG':
      return { ...state, isDialogOpen: true, isEditDialog: false };

    case 'OPEN_EDIT_DIALOG':
      return {
        ...state,
        isDialogOpen: true,
        isEditDialog: true,
        selectedCategory: action.selectedCategory,
      };

    case 'CLOSE_DIALOG':
      return {
        ...state,
        isDialogOpen: false,
        isEditDialog: false,
        selectedCategory: null,
      };

    default:
      throw new Error();
  }
}

const initialState = {
  orderBy: '',
  searchText: '',
  indexedLabels: {},
  filteredData: null,
  isDialogOpen: false,
  isEditDialog: false,
  selectedCategory: null,
  orderDescending: false,
};

function Status() {
  const [state, setState] = useReducer(reducer, initialState);

  const {
    orderBy,
    searchText,
    filteredData,
    isDialogOpen,
    isEditDialog,
    indexedLabels,
    orderDescending,
    selectedCategory,
  } = state;

  const pageLayout = useRef(null);
  const { labelHandle, filterHandle, folderHandle } = useParams();

  const { t } = useTranslation('status');

  const { data: labels = [], isLoading: isLabelsLoading } = useCategoryLabels();
  const { data: status, isLoading } = useStatus({
    labelHandle,
    filterHandle,
    folderHandle,
  });

  const [saveCategory] = useSaveCategory();

  useEffect(() => {
    console.log('useEffect', { searchText, orderBy, orderDescending });

    setState({
      type: 'SET_FILTERED_DATA',
      orderBy,
      status,
      searchText,
      orderDescending,
    });
  }, [orderBy, orderDescending, status, searchText]);

  useEffect(() => {
    if (status?.length > 0 && labels.length > 0) {
      console.log('indexedLabels useEffect');

      const indexedLabels = {};
      labels.forEach((label) => {
        indexedLabels[label.id] = label;
      });
      setState({ type: 'SET_INDEXED_LABELS', indexedLabels });
    }
  }, [status?.length, labels]);

  const handleUpdateCategory = (category) => {
    return saveCategory({
      category,
      labelHandle,
      filterHandle,
      folderHandle,
      isEdit: true,
    });
  };

  const handleToggleOrderDescending = () => {
    setState({ type: 'TOGGLE_ORDER_DESCENDING' });
  };

  const handleSetSearchText = (searchText) => {
    setState({ type: 'SET_SEARCH_TEXT', searchText });
  };

  const handleSetOrderBy = (orderBy) => {
    setState({ type: 'SET_ORDER_BY', orderBy });
  };

  const handleOpenAddDialog = () => {
    setState({ type: 'OPEN_ADD_DIALOG' });
  };

  const handleOpenEditDialog = (selectedCategory) => {
    setState({ type: 'OPEN_EDIT_DIALOG', selectedCategory });
  };

  const handleCloseDialog = () => {
    setState({ type: 'CLOSE_DIALOG' });
  };

  const handleSubmit = (category) => {
    return saveCategory({
      category,
      labelHandle,
      filterHandle,
      folderHandle,
      isEdit: isEditDialog,
    });
  };

  if (!filteredData) {
    return null;
  }

  return (
    <>
      {isLoading && <Spinner />}
      <FusePageCarded
        classes={{
          root: 'w-full',
          header: 'items-center min-h-72 h-72 sm:h-136 sm:min-h-136',
        }}
        header={
          <StatusHeader
            t={t}
            searchText={searchText}
            pageLayout={pageLayout}
            onOpenAddDialog={handleOpenAddDialog}
            onSearchTextChange={handleSetSearchText}
          />
        }
        contentToolbar={
          <StatusToolbar
            t={t}
            orderBy={orderBy}
            orderDescending={orderDescending}
            onOrderByChange={handleSetOrderBy}
            onToggleOrderDescending={handleToggleOrderDescending}
          />
        }
        content={
          <CategoryList
            t={t}
            isLoading={isLoading}
            status={filteredData}
            indexedLabels={indexedLabels}
            onUpdate={handleUpdateCategory}
            onOpenEditDialog={handleOpenEditDialog}
          />
        }
        leftSidebarHeader={<StatusSidebarHeader t={t} />}
        leftSidebarContent={
          <StatusSidebarContent
            t={t}
            labels={labels}
            isLabelsLoading={isLabelsLoading}
          />
        }
        ref={pageLayout}
        innerScroll
      />

      {isDialogOpen && (
        <CategoryFormDialog
          t={t}
          labels={labels}
          edit={isEditDialog}
          onSubmit={handleSubmit}
          data={selectedCategory}
          onClose={handleCloseDialog}
          indexedLabels={indexedLabels}
        />
      )}
    </>
  );
}

export default Status;
