import { useEffect, useMemo, useState } from 'react';
import qs from 'qs';
import Fab from '@material-ui/core/Fab';
import Icon from '@material-ui/core/Icon';
import Spinner from 'app/components/Spinner';
import ProductsHeader from './ProductsHeader';
import useProducts from '../hooks/useProducts';
import { useTranslation } from 'react-i18next';
import DataTable from 'app/components/DataTable';
import { makeStyles } from '@material-ui/core/styles';
import IconButton from '@material-ui/core/IconButton';
import FusePageCarded from '@fuse/core/FusePageCarded';
import UploadIcon from '@material-ui/icons/CloudUpload';
import { useLocation, useHistory } from 'react-router-dom';
import useDeleteProducts from '../hooks/useDeleteProducts';
import useDataTable from 'app/components/DataTable/useDataTable';
import DataTableCheckbox from 'app/components/DataTable/DataTableCheckbox';

const useStyles = makeStyles((theme) => ({
  fab: {
    position: 'absolute',
    bottom: theme.spacing(2),
    right: theme.spacing(2),
  },
}));

function Products() {
  const classes = useStyles();
  const { t } = useTranslation('products');

  const history = useHistory();
  const { search: locationSearch } = useLocation();

  const urlParams = qs.parse(locationSearch, {
    ignoreQueryPrefix: true,
  });

  const [queryPerPage, setQueryPerPage] = useState(
    parseInt(urlParams.perPage) || 5
  );
  const [queryPage, setQueryPage] = useState(parseInt(urlParams.page) || 1);
  const [querySortBy, setQuerySortBy] = useState(urlParams.sortBy);
  const [querySortDir, setQuerySortDir] = useState(urlParams.sortDir);
  const [querySearch, setQuerySearch] = useState(urlParams.search);

  const {
    isLoading,
    isFetching,
    resolvedData = { count: 0, products: [], pageCount: 0 },
  } = useProducts({
    page: queryPage,
    search: querySearch,
    sortBy: querySortBy,
    perPage: queryPerPage,
    sortDir: querySortDir,
  });

  const [deleteProducts] = useDeleteProducts();

  const columns = useMemo(
    () => [
      {
        id: 'selection',
        header: ({
          selectedRows,
          isAllRowsSelected,
          isIndeterminateSelect,
          onToggleAllSelectedRow,
        }) => {
          const selectedRowIds = Object.keys(selectedRows);

          return (
            <div className="flex">
              <DataTableCheckbox
                className="mx-10"
                checked={isAllRowsSelected}
                indeterminate={isIndeterminateSelect}
                onChange={(e) => onToggleAllSelectedRow(e.target.checked)}
              />

              {selectedRowIds.length > 0 && (
                <div className="flex items-center justify-center h-64 mx-10">
                  <IconButton
                    onClick={(e) => {
                      e.stopPropagation();
                      deleteProducts(selectedRowIds).then(() => {
                        onToggleAllSelectedRow(false);
                      });
                    }}
                  >
                    <Icon color="error">delete</Icon>
                  </IconButton>
                </div>
              )}
            </div>
          );
        },
        accessor: ({ row, isSelected, handleToggleSelectedRow }) => (
          <div>
            <DataTableCheckbox
              className="mx-10"
              checked={isSelected}
              onChange={(e) =>
                handleToggleSelectedRow(row.id, e.target.checked)
              }
            />
          </div>
        ),
        headerClassName: 'w-136',
        disablePadding: true,
      },
      // {
      //   Header: ({ selectedFlatRows }) => {
      //     const selectedRowIds = selectedFlatRows.map((row) => row.original.id);
      //     return (
      //       selectedFlatRows.length > 0 && (
      //         <ProductsMultiSelectMenu selectedIds={selectedRowIds} />
      //       )
      //     );
      //   },
      //   accessor: 'images',
      //   Cell: ({ row }) => (
      //     <img
      //       className="rounded"
      //       alt={row.original.name}
      //       src={row.original.images[0] ? row.original.images[0].url : null}
      //     />
      //   ),
      //   className: 'w-52 px-4 md:px-0',
      //   width: 64,
      //   sortable: false,
      // },
      { header: t('NAME'), accessor: 'name', sortable: true },
      { header: t('CATEGORY'), accessor: 'categories', sortable: true },
      { header: t('PRICE'), accessor: 'priceTaxIncl', sortable: true },
      { header: t('QUANTITY'), accessor: 'quantity', sortable: true },
      {
        id: 'active',
        header: t('ACTIVE'),
        sortable: true,
        accessor: ({ row }) =>
          row.active ? (
            <Icon className="text-green text-20">check_circle</Icon>
          ) : (
            <Icon className="text-red text-20">remove_circle</Icon>
          ),
      },
    ],
    [t, deleteProducts]
  );

  const {
    page,
    sortBy,
    search,
    perPage,
    sortDir,
    handleSearch,
    appliedFilters,
    ...tableInstance
  } = useDataTable({
    columns,
    initialPage: queryPage,
    initialSortBy: querySortBy,
    initialSearch: querySearch,
    data: resolvedData?.products,
    initialPerPage: queryPerPage,
    initialSortDir: querySortDir,
  });

  useEffect(() => {
    setQueryPage(page);
  }, [page]);

  useEffect(() => {
    setQueryPerPage(perPage);
  }, [perPage]);

  useEffect(() => {
    setQuerySearch(search);
  }, [search]);

  useEffect(() => {
    if (sortBy !== querySortBy) {
      setQuerySortBy(sortBy);
    }

    if (sortDir !== querySortDir) {
      setQuerySortDir(sortDir);
    }
  }, [sortBy, sortDir, querySortBy, querySortDir]);

  const handleNavigateToProduct = (row) => {
    history.push(`/apps/e-commerce/products/${row.id}`);
  };

  return (
    <FusePageCarded
      classes={{
        content: 'flex',
        contentCard: 'overflow-hidden',
        header: 'min-h-72 h-72 sm:h-136 sm:min-h-136',
      }}
      header={<ProductsHeader onSearch={handleSearch} />}
      content={
        <>
          {isLoading && <Spinner />}
          <DataTable
            hover
            page={page}
            stickyHeader
            sortBy={sortBy}
            sortDir={sortDir}
            perPage={perPage}
            isLoading={isFetching}
            count={resolvedData?.count}
            onRowClick={handleNavigateToProduct}
            paginationClasses={{ toolbar: 'ltr:pr-52 rtl:pl-52' }}
            {...tableInstance}
          />
          <Fab
            color="primary"
            className={classes.fab}
            aria-label="upload excel file"
          >
            <UploadIcon />
          </Fab>
        </>
      }
      innerScroll
    />
  );
}

export default Products;
