import * as yup from 'yup';
import PropTypes from 'prop-types';
import Icon from '@material-ui/core/Icon';
import { getImageUrl } from 'app/utils/url';
import AppBar from '@material-ui/core/AppBar';
import Dialog from '@material-ui/core/Dialog';
import { useTranslation } from 'react-i18next';
import ImageBox from 'app/components/ImageBox';
import Toolbar from '@material-ui/core/Toolbar';
import Divider from '@material-ui/core/Divider';
import Checkbox from '@material-ui/core/Checkbox';
import TextField from '@material-ui/core/TextField';
import Typography from '@material-ui/core/Typography';
import { useForm, Controller } from 'react-hook-form';
import { yupResolver } from '@hookform/resolvers/yup';
import IconButton from '@material-ui/core/IconButton';
import { amber, red } from '@material-ui/core/colors';
import { makeStyles } from '@material-ui/core/styles';
import LoadingButton from 'app/components/LoadingButton';
import DialogActions from '@material-ui/core/DialogActions';
import DialogContent from '@material-ui/core/DialogContent';
import DialogCloseButton from 'app/components/DialogCloseButton';
import FormControlLabel from '@material-ui/core/FormControlLabel';
import useDeleteTreeCategory from '../hooks/useDeleteTreeCategory';

const useStyles = makeStyles((theme) => ({
  dialogContent: {
    padding: '0 !important',
  },
}));

const validationSchema = yup.object().shape({
  title: yup.string().required(),
});

const defaultFormState = {
  mail: '',
  title: '',
  notes: '',
  phone: '',
  labels: [],
  images: {},
  type: null,
  iconSrc: '',
  address: '',
  imageSrc: '',
  children: [],
  cluster: null,
  expanded: true,
  starred: false,
  disable: false,
  deleted: false,
  completed: false,
  important: false,
  labelsObject: [],
  dueDate: new Date(),
  categorylabel: null,
  startDate: new Date(),
};

function TreeCategoryFormDialog({ onClose, onSubmit, edit, selectedCategory }) {
  const classes = useStyles();

  const {
    watch,
    reset,
    errors,
    control,
    register,
    setValue,
    getValues,
    handleSubmit,
    formState: { isSubmitting },
  } = useForm({
    shouldUnregister: false,
    resolver: yupResolver(validationSchema),
    defaultValues: edit ? selectedCategory?.node : defaultFormState,
  });

  const { t } = useTranslation('products');

  const [
    deleteTreeCategory,
    { isLoading: isDeleting },
  ] = useDeleteTreeCategory();

  const handleToggleImportant = () => {
    const isImportant = getValues('important');
    setValue('important', !isImportant);
  };

  const handleToggleStarred = () => {
    const isStarred = getValues('starred');
    setValue('starred', !isStarred);
  };

  const handleUploadChange = (e, fieldName) => {
    const file = e.target.files[0];

    setValue(fieldName + 'Src', URL.createObjectURL(file));
    setValue(fieldName + 'File', file);

    e.target.value = null;
  };

  const handleDeleteImage = (fieldName) => {
    setValue(fieldName + 'Src', '');
    setValue(fieldName + 'File', '');
  };

  const handleDeleteCategory = () => {
    deleteTreeCategory(selectedCategory.path).then(() => {
      onClose();
    });
  };

  const handleSubmitForm = (data) => {
    const { path = null, treeIndex = null } = selectedCategory || {};
    return onSubmit(data, path, treeIndex).then((response) => {
      if (edit) {
        reset(response);
      } else {
        onClose();
      }
    });
  };

  const formValues = watch([
    'starred',
    'iconSrc',
    'imageSrc',
    'iconFile',
    'important',
    'imageFile',
  ]);

  return (
    <Dialog
      fullWidth
      open={true}
      maxWidth="sm"
      onClose={onClose}
      disableBackdropClick
      classes={{ paper: 'rounded-8' }}
    >
      <AppBar position="static" elevation={1}>
        <Toolbar className="flex w-full">
          <Typography variant="subtitle1" color="inherit">
            {edit ? t('EDIT_CATEGORY') : t('ADD_NEW_CATEGORY')}
          </Typography>
        </Toolbar>
        <DialogCloseButton onClick={onClose} />
      </AppBar>

      <form
        noValidate
        onSubmit={handleSubmit(handleSubmitForm)}
        className="flex flex-col md:overflow-hidden"
      >
        <DialogContent classes={{ root: classes.dialogContent }}>
          <div className="mb-16">
            <div className="flex items-center justify-between p-12">
              <div className="flex">
                <FormControlLabel
                  control={
                    <Controller
                      name="disable"
                      render={(props) => (
                        <Checkbox
                          checked={props.value}
                          onChange={(e) => props.onChange(e.target.checked)}
                        />
                      )}
                      control={control}
                    />
                  }
                  label={t('DISABLED')}
                />
              </div>

              <div
                className="flex items-center justify-start"
                aria-label="Toggle star"
              >
                <IconButton onClick={handleToggleImportant}>
                  {formValues.important ? (
                    <Icon style={{ color: red[500] }}>error</Icon>
                  ) : (
                    <Icon>error_outline</Icon>
                  )}
                </IconButton>

                <IconButton onClick={handleToggleStarred}>
                  {formValues.starred ? (
                    <Icon style={{ color: amber[500] }}>star</Icon>
                  ) : (
                    <Icon>star_outline</Icon>
                  )}
                </IconButton>
              </div>
            </div>
            <Divider className="mx-24" />
          </div>

          <div className="px-16 sm:px-24">
            <TextField
              required
              fullWidth
              autoFocus
              type="text"
              name="title"
              className="mb-16"
              label={t('TITLE')}
              variant="outlined"
              inputRef={register}
              error={!!errors?.title}
              helperText={errors?.title?.message}
            />

            {/* TODO: types */}

            <TextField
              rows="3"
              fullWidth
              multiline
              type="text"
              name="notes"
              className="mb-16"
              variant="outlined"
              inputRef={register}
              label={t('DESCRIPTION')}
            />

            <TextField
              fullWidth
              type="tel"
              name="phone"
              className="mb-16"
              label={t('PHONE')}
              variant="outlined"
              inputRef={register}
            />

            <TextField
              fullWidth
              name="mail"
              type="email"
              className="mb-16"
              label={t('EMAIL')}
              variant="outlined"
              inputRef={register}
            />

            <TextField
              rows="3"
              fullWidth
              multiline
              type="text"
              name="address"
              className="mb-16"
              variant="outlined"
              inputRef={register}
              label={t('ADDRESS')}
            />

            <div className="flex items-center flex-wrap -mx-16 mb-32">
              <ImageBox
                t={t}
                accept="image/*"
                name="background"
                onDelete={() => handleDeleteImage('image')}
                onChange={(e) => handleUploadChange(e, 'image')}
                imageSource={
                  formValues.imageFile
                    ? formValues.imageSrc
                    : getImageUrl(formValues.imageSrc)
                }
              />

              <ImageBox
                t={t}
                name="icon"
                accept="image/png"
                onDelete={() => handleDeleteImage('icon')}
                onChange={(e) => handleUploadChange(e, 'icon')}
                imageSource={
                  formValues.iconFile
                    ? formValues.iconSrc
                    : getImageUrl(formValues.iconSrc)
                }
              />
            </div>
          </div>
        </DialogContent>

        <DialogActions className="justify-between p-8">
          <div className="px-16">
            <LoadingButton
              type="submit"
              color="primary"
              variant="contained"
              isLoading={isSubmitting || isDeleting}
              className="whitespace-no-wrap normal-case"
            >
              {t('SAVE')}
            </LoadingButton>
          </div>
          {edit && (
            <IconButton
              disabled={isDeleting}
              className="min-w-auto"
              onClick={handleDeleteCategory}
            >
              <Icon>delete</Icon>
            </IconButton>
          )}
        </DialogActions>
      </form>
    </Dialog>
  );
}

TreeCategoryFormDialog.propTypes = {
  edit: PropTypes.bool,
  selectedCategory: PropTypes.object,
  onClose: PropTypes.func.isRequired,
  onSubmit: PropTypes.func.isRequired,
};

export default TreeCategoryFormDialog;
