import _ from '@lodash';
import clsx from 'clsx';
import PropTypes from 'prop-types';
import List from '@material-ui/core/List';
import Icon from '@material-ui/core/Icon';
import PropertyChip from './PropertyChip';
import FuseAnimate from '@fuse/core/FuseAnimate';
import ListItem from '@material-ui/core/ListItem';
import { amber, red } from '@material-ui/core/colors';
import IconButton from '@material-ui/core/IconButton';
import { makeStyles } from '@material-ui/core/styles';
import Typography from '@material-ui/core/Typography';
import FuseAnimateGroup from '@fuse/core/FuseAnimateGroup';

const useStyles = makeStyles({
  item: {
    '&.completed': {
      background: 'rgba(0,0,0,0.03)',
      '& .property-title, & .property-notes': {
        textDecoration: 'line-through',
      },
    },
  },
});

function PropertyList({
  t,
  onUpdate,
  isLoading,
  properties,
  indexedLabels,
  onOpenEditDialog,
}) {
  const classes = useStyles();

  if (!isLoading && properties.length === 0) {
    return (
      <FuseAnimate delay={100}>
        <div className="flex flex-1 items-center justify-center h-full">
          <Typography color="textSecondary" variant="h5">
            {t('THERE_ARE_NO_PROPERTIES')}
          </Typography>
        </div>
      </FuseAnimate>
    );
  }

  return (
    <List className="p-0">
      <FuseAnimateGroup
        enter={{
          animation: 'transition.slideUpBigIn',
        }}
      >
        {properties.map((property) => (
          <ListItem
            dense
            button
            key={property.id}
            className={clsx(
              classes.item,
              { completed: property.completed },
              'border-solid border-b-1 py-16 px-0 sm:px-8'
            )}
            onClick={(ev) => {
              ev.preventDefault();
              onOpenEditDialog(property);
            }}
          >
            <div className="flex flex-1 flex-col relative overflow-hidden px-8">
              <Typography
                variant="subtitle1"
                className="property-title truncate"
                color={property.completed ? 'textSecondary' : 'inherit'}
              >
                {property.title}
              </Typography>

              <Typography
                color="textSecondary"
                className="property-notes truncate"
              >
                {_.truncate(
                  property.description.replace(/<(?:.|\n)*?>/gm, ''),
                  {
                    length: 180,
                  }
                )}
              </Typography>

              <div className={clsx(classes.labels, 'flex -mx-2')}>
                {property.labels.map((labelId) => {
                  const label = indexedLabels[labelId];
                  return label ? (
                    <PropertyChip
                      key={labelId}
                      title={label.title}
                      className="mx-2 mt-4"
                      color={label.color}
                    />
                  ) : null;
                })}
              </div>
            </div>

            <div className="px-8">
              <IconButton
                onClick={(ev) => {
                  ev.preventDefault();
                  ev.stopPropagation();
                  onUpdate({
                    ...property,
                    important: !property.important,
                  });
                }}
              >
                {property.important ? (
                  <Icon style={{ color: red[500] }}>error</Icon>
                ) : (
                  <Icon>error_outline</Icon>
                )}
              </IconButton>
              <IconButton
                onClick={(ev) => {
                  ev.preventDefault();
                  ev.stopPropagation();
                  onUpdate({
                    ...property,
                    starred: !property.starred,
                  });
                }}
              >
                {property.starred ? (
                  <Icon style={{ color: amber[500] }}>star</Icon>
                ) : (
                  <Icon>star_outline</Icon>
                )}
              </IconButton>
            </div>
          </ListItem>
        ))}
      </FuseAnimateGroup>
    </List>
  );
}

PropertyList.propTypes = {
  t: PropTypes.func.isRequired,
  onUpdate: PropTypes.func.isRequired,
  isLoading: PropTypes.bool.isRequired,
  properties: PropTypes.array.isRequired,
  indexedLabels: PropTypes.object.isRequired,
  onOpenEditDialog: PropTypes.func.isRequired,
};

export default PropertyList;
