import clsx from 'clsx';
import { useState } from 'react';
import PropTypes from 'prop-types';
import FuseUtils from '@fuse/utils';
import CropDialog from './CropDialog';
import Icon from '@material-ui/core/Icon';
import Lightbox from 'react-image-lightbox';
import CropIcon from '@material-ui/icons/Crop';
import Tooltip from '@material-ui/core/Tooltip';
import { useFieldArray } from 'react-hook-form';
import DeleteIcon from '@material-ui/icons/Delete';
import IconButton from '@material-ui/core/IconButton';
import { makeStyles } from '@material-ui/core/styles';
import { orange, red, green } from '@material-ui/core/colors';
import 'react-image-lightbox/style.css';

const useStyles = makeStyles((theme) => ({
  productImageFeaturedStar: {
    position: 'absolute',
    top: 0,
    right: 0,
    color: orange[400],
    opacity: 0,
  },
  productImageDelete: {
    position: 'absolute',
    bottom: 0,
    right: 0,
    color: red[400],
    opacity: 0,
  },
  productImageCrop: {
    position: 'absolute',
    bottom: 0,
    left: 0,
    color: green[400],
    opacity: 0,
  },
  productImageUpload: {
    transitionProperty: 'box-shadow',
    transitionDuration: theme.transitions.duration.short,
    transitionTimingFunction: theme.transitions.easing.easeInOut,
  },
  productImageItem: {
    transitionProperty: 'box-shadow',
    transitionDuration: theme.transitions.duration.short,
    transitionTimingFunction: theme.transitions.easing.easeInOut,
    '&:hover': {
      '& $productImageFeaturedStar': {
        opacity: 0.8,
      },
      '& $productImageDelete': {
        opacity: 0.8,
      },
      '& $productImageCrop': {
        opacity: 0.8,
      },
    },
    '&.featured': {
      pointerEvents: 'none',
      boxShadow: theme.shadows[3],
      '& $productImageFeaturedStar': {
        opacity: 1,
      },
      '&:hover $productImageFeaturedStar': {
        opacity: 1,
      },
      '& $productImageDelete': {
        opacity: 1,
      },
      '&:hover $productImageDelete': {
        opacity: 1,
      },
      '& $productImageCrop': {
        opacity: 1,
      },
      '&:hover $productImageCrop': {
        opacity: 1,
      },
    },
  },
}));

const ProductMedia = ({ t, control, setValue, watch }) => {
  const [isCropDialogOpen, setIsCropDialogOpen] = useState(false);
  const [selectedImageForCrop, setSelectedImageForCrop] = useState(null);

  const [lightboxImageIndex, setLightboxImageIndex] = useState(null);
  const [isLightboxOpen, setIsLightboxOpen] = useState(false);

  const classes = useStyles();

  const { append, remove, fields: images } = useFieldArray({
    control, // control props comes from useForm (optional: if you are using FormContext)
    name: 'images', // unique name for your Field Array
    keyName: 'index',
    // keyName: "id", default to "id", you can change the key name
  });

  const handleDelete = (e, index) => {
    e.stopPropagation();
    remove(index);
  };

  const handleOpenCropDialog = (e, media) => {
    e.stopPropagation();
    setSelectedImageForCrop(media);
    setIsCropDialogOpen(true);
  };

  const handleCloseCropDialog = () => {
    setIsCropDialogOpen(false);
    setSelectedImageForCrop(null);
  };

  const handleUploadChange = (e) => {
    [...e.target.files].forEach((file) => {
      append({
        id: FuseUtils.generateGUID(),
        url: URL.createObjectURL(file),
        type: 'image',
        file,
        __isChanged__: true,
      });
    });

    e.target.value = null;
  };

  const handleSaveCroppedImage = (croppedImage) => {
    setValue(
      'images',
      images.map((item) => {
        const { index, ...image } = item;

        if (index === selectedImageForCrop.index) {
          return {
            ...image,
            url: croppedImage,
            file: new File([croppedImage], 'name'),
            __isChanged__: true,
          };
        }
        return image;
      }),
      { shouldDirty: true }
    );

    handleCloseCropDialog();
  };

  const handleOpenLightbox = (index) => {
    setIsLightboxOpen(true);
    setLightboxImageIndex(index);
  };

  const handleCloseLightbox = () => {
    setIsLightboxOpen(false);
    setLightboxImageIndex(null);
  };

  const handleSetFeaturedImage = (e, image) => {
    e.stopPropagation();
    setValue('featuredImageId', image.id, { shouldDirty: true });
  };

  const featuredImageId = watch('featuredImageId');

  return (
    <>
      <div className="flex justify-center sm:justify-start flex-wrap -mx-8">
        <label
          htmlFor="button-file"
          className={clsx(
            classes.productImageUpload,
            'flex items-center justify-center relative w-128 h-128 rounded-8 mx-8 mb-16 overflow-hidden cursor-pointer shadow-1 hover:shadow-5'
          )}
        >
          <input
            type="file"
            accept="image/*"
            id="button-file"
            className="hidden"
            onChange={handleUploadChange}
          />
          <Icon fontSize="large" color="action">
            cloud_upload
          </Icon>
        </label>
        {images.map((media, index) => (
          <div
            key={media.index}
            role="button"
            tabIndex={0}
            className={clsx(
              classes.productImageItem,
              'flex items-center justify-center relative w-128 h-128 rounded-8 mx-8 mb-16 overflow-hidden cursor-pointer shadow-1 hover:shadow-5',
              media.id === featuredImageId && 'featured'
            )}
            onClick={(e) => handleSetFeaturedImage(e, media)}
          >
            <Icon className={classes.productImageFeaturedStar}>star</Icon>
            <Tooltip title={t('DELETE')}>
              <IconButton
                aria-label={t('DELETE')}
                onClick={(e) => handleDelete(e, index)}
                className={classes.productImageDelete}
              >
                <DeleteIcon />
              </IconButton>
            </Tooltip>

            <Tooltip title={t('CROP')}>
              <IconButton
                aria-label={t('CROP')}
                className={classes.productImageCrop}
                onClick={(e) => handleOpenCropDialog(e, media)}
              >
                <CropIcon />
              </IconButton>
            </Tooltip>

            <img
              src={media.url}
              alt={t('PRODUCT')}
              name={`test[${index}].value`}
              className="max-w-none w-auto h-full"
              onClick={() => handleOpenLightbox(index)}
            />
          </div>
        ))}
      </div>
      {isCropDialogOpen && (
        <CropDialog
          isOpen={isCropDialogOpen}
          onClose={handleCloseCropDialog}
          onSave={handleSaveCroppedImage}
          imageUrl={selectedImageForCrop?.url}
        />
      )}

      {isLightboxOpen && (
        <Lightbox
          onCloseRequest={handleCloseLightbox}
          mainSrc={images[lightboxImageIndex].url}
          nextSrc={images[images[lightboxImageIndex + 1] % images.length]}
          prevSrc={
            images[(lightboxImageIndex + images.length - 1) % images.length]
          }
          onMovePrevRequest={() =>
            setLightboxImageIndex(
              (lightboxImageIndex + images.length - 1) % images.length
            )
          }
          onMoveNextRequest={() =>
            setLightboxImageIndex((lightboxImageIndex + 1) % images.length)
          }
        />
      )}
    </>
  );
};

ProductMedia.propTypes = {
  t: PropTypes.func.isRequired,
  watch: PropTypes.func.isRequired,
  setValue: PropTypes.func.isRequired,
  control: PropTypes.object.isRequired,
};

export default ProductMedia;
