import { useState } from 'react';
import PropTypes from 'prop-types';
import Dialog from '@material-ui/core/Dialog';
import AppBar from '@material-ui/core/AppBar';
import Button from '@material-ui/core/Button';
import Toolbar from '@material-ui/core/Toolbar';
import useCategories from '../../hooks/useCategories';
import Typography from '@material-ui/core/Typography';
import FuseChipSelect from '@fuse/core/FuseChipSelect';
import DialogActions from '@material-ui/core/DialogActions';
import DialogContent from '@material-ui/core/DialogContent';
import useCategoryLabels from '../../hooks/useCategoryLabels';
import DialogCloseButton from 'app/components/DialogCloseButton';

const CategoryDialog = ({ t, isOpen, onClose, onSelectCategory }) => {
  const [selectedCategory, setSelectedCategory] = useState(null);
  const [selectedCategoryLabel, setSelectedCategoryLabel] = useState(null);
  const [categoryOptions, setCategoryOptions] = useState([]);

  const { data: categories = [] } = useCategories();
  const { data: categoryLabels = [] } = useCategoryLabels();

  const handleCategoryLabelChange = (value) => {
    value = !value || value?.length === 0 ? null : value;

    setSelectedCategoryLabel(value);

    if (value) {
      setCategoryOptions(
        categories.filter(({ labels }) => labels.includes(value.id))
      );
    } else {
      setCategoryOptions([]);
      setSelectedCategory(null);
    }
  };

  const handleSelectCategory = () => {
    const category = {
      categoryLabelId: selectedCategoryLabel.id,
      value: selectedCategory
        ? selectedCategoryLabel.title + ' / ' + selectedCategory.title
        : selectedCategoryLabel.title,
    };

    if (selectedCategory) {
      category.id = selectedCategory.id;
      category.title = selectedCategory.title;
    }

    onSelectCategory(category);
    onClose();
    setSelectedCategoryLabel(null);
    setSelectedCategory(null);
  };

  return (
    <Dialog
      fullWidth
      open={isOpen}
      maxWidth="sm"
      onClose={onClose}
      classes={{ paper: 'rounded-8' }}
    >
      <AppBar position="static" elevation={1}>
        <Toolbar className="flex w-full">
          <Typography variant="subtitle1" color="inherit">
            {t('ADD_CATEGORY')}
          </Typography>
        </Toolbar>
        <DialogCloseButton onClick={onClose} />
      </AppBar>

      <DialogContent classes={{ root: 'p-28 mt-16' }}>
        <FuseChipSelect
          isClearable
          className="mb-24"
          isSearchable={true}
          options={categoryLabels}
          placeholder={t('SEARCH')}
          value={selectedCategoryLabel}
          onChange={handleCategoryLabelChange}
          getOptionValue={(option) => option.id}
          getOptionLabel={(option) => option.title}
          textFieldProps={{
            label: t('LABEL'),
            variant: 'outlined',
            InputLabelProps: { shrink: true },
          }}
        />

        <FuseChipSelect
          isClearable
          className="mb-24"
          isSearchable={true}
          value={selectedCategory}
          options={categoryOptions}
          placeholder={t('SEARCH')}
          getOptionValue={(option) => option.id}
          getOptionLabel={(option) => option.title}
          onChange={(value) => setSelectedCategory(value)}
          textFieldProps={{
            label: t('LABEL'),
            variant: 'outlined',
            InputLabelProps: { shrink: true },
          }}
        />
      </DialogContent>

      <DialogActions className="justify-between p-8">
        <div className="px-16">
          <Button
            type="button"
            color="primary"
            variant="contained"
            onClick={handleSelectCategory}
            disabled={!selectedCategoryLabel}
          >
            {t('ADD')}
          </Button>
        </div>
      </DialogActions>
    </Dialog>
  );
};

CategoryDialog.propTypes = {
  t: PropTypes.func.isRequired,
  isOpen: PropTypes.bool.isRequired,
  onClose: PropTypes.func.isRequired,
  onSelectCategory: PropTypes.func.isRequired,
};

export default CategoryDialog;
