import { useEffect, useMemo, useState } from 'react';
import qs from 'qs';
import Fab from '@material-ui/core/Fab';
import Icon from '@material-ui/core/Icon';
import Spinner from 'app/components/Spinner';
import OrderListHeader from './OrderListHeader';
import useOrderList from '../hooks/useOrderList';
import { useTranslation } from 'react-i18next';
import DataTable from 'app/components/DataTable';
import { makeStyles } from '@material-ui/core/styles';
import IconButton from '@material-ui/core/IconButton';
import FusePageCarded from '@fuse/core/FusePageCarded';
import UploadIcon from '@material-ui/icons/CloudUpload';
import { useLocation, useHistory } from 'react-router-dom';
import useDeleteOrderList from '../hooks/useDeleteOrderList';
import useDataTable from 'app/components/DataTable/useDataTable';
import DataTableCheckbox from 'app/components/DataTable/DataTableCheckbox';

const useStyles = makeStyles((theme) => ({
  fab: {
    position: 'absolute',
    bottom: theme.spacing(2),
    right: theme.spacing(2),
  },
}));

function OrderList() {
  const classes = useStyles();
  const { t } = useTranslation('orders');

  const history = useHistory();
  const { search: locationSearch } = useLocation();

  const urlParams = qs.parse(locationSearch, {
    ignoreQueryPrefix: true,
  });

  const [queryPerPage, setQueryPerPage] = useState(
    parseInt(urlParams.perPage) || 5
  );
  const [queryPage, setQueryPage] = useState(parseInt(urlParams.page) || 1);
  const [querySortBy, setQuerySortBy] = useState(urlParams.sortBy);
  const [querySortDir, setQuerySortDir] = useState(urlParams.sortDir);
  const [querySearch, setQuerySearch] = useState(urlParams.search);

  const {
    isLoading,
    isFetching,
    resolvedData = { count: 0, orders: [], pageCount: 0 },
  } = useOrderList({
    page: queryPage,
    search: querySearch,
    sortBy: querySortBy,
    perPage: queryPerPage,
    sortDir: querySortDir,
  });

  const [deleteOrderList] = useDeleteOrderList();
 
  const columns = useMemo(
    () => [
      {
        id: 'selection',
        header: ({
          selectedRows,
          isAllRowsSelected,
          isIndeterminateSelect,
          onToggleAllSelectedRow,
        }) => {
          const selectedRowIds = Object.keys(selectedRows);

          return (
            <div className="flex">
              <DataTableCheckbox
                className="mx-10"
                checked={isAllRowsSelected}
                indeterminate={isIndeterminateSelect}
                onChange={(e) => onToggleAllSelectedRow(e.target.checked)}
              />

              {selectedRowIds.length > 0 && (
                <div className="flex items-center justify-center h-64 mx-10">
                  <IconButton
                    onClick={(e) => {
                      e.stopPropagation();
                      deleteOrderList(selectedRowIds).then(() => {
                        onToggleAllSelectedRow(false);
                      });
                    }}
                  >
                    <Icon color="error">delete</Icon>
                  </IconButton>
                </div>
              )}
            </div>
          );
        },
        accessor: ({ row, isSelected, handleToggleSelectedRow }) => (
          <div>
            <DataTableCheckbox
              className="mx-10"
              checked={isSelected}
              onChange={(e) =>
                handleToggleSelectedRow(row.id, e.target.checked)
              }
            />
          </div>
        ),
        headerClassName: 'w-136',
        disablePadding: true,
      },
      {
        header: t('CUSTOMER_NAME'),
        sortable: true,
        accessor: ({ row }) =>
          row.customer.firstName + ' ' + row.customer.lastName,
      },
      {
        header: t('payment_method'),
        accessor: ({ row }) => row.gateway.title,
        sortable: true,
      },
      { header: t('PRICE'), accessor: 'total', sortable: true },
      {
        header: t('STATUS'),
        sortable: true,
        accessor: ({ row }) =>
          row.status.length ? row.status[row.status.length - 1].name : '',
        id: 'status',
      },
      { header: t('DATE'), accessor: 'fdate', sortable: true },
      { header: t('DESCRIPTION'), accessor: 'description', sortable: true },
      {
        id: 'active',
        header: t('ACTIVE'),
        sortable: true,
        accessor: ({ row }) =>
          row.active ? (
            <Icon className="text-green text-20">check_circle</Icon>
          ) : (
            <Icon className="text-red text-20">remove_circle</Icon>
          ),
      },
    ],
    [t, deleteOrderList]
  );

  const {
    page,
    sortBy,
    search,
    perPage,
    sortDir,
    handleSearch,
    appliedFilters,
    ...tableInstance
  } = useDataTable({
    columns,
    initialPage: queryPage,
    initialSortBy: querySortBy,
    initialSearch: querySearch,
    data: resolvedData?.orders,
    initialPerPage: queryPerPage,
    initialSortDir: querySortDir,
  });

  useEffect(() => {
    setQueryPage(page);
  }, [page]);

  useEffect(() => {
    setQueryPerPage(perPage);
  }, [perPage]);

  useEffect(() => {
    setQuerySearch(search);
  }, [search]);

  useEffect(() => {
    if (sortBy !== querySortBy) {
      setQuerySortBy(sortBy);
    }

    if (sortDir !== querySortDir) {
      setQuerySortDir(sortDir);
    }
  }, [sortBy, sortDir, querySortBy, querySortDir]);

  const handleNavigateToOrder = (row) => {
    history.push(`/apps/e-laundry/ordersview/${row.id}`);
  };
  
  return (
    <FusePageCarded
      classes={{
        content: 'flex',
        contentCard: 'overflow-hidden',
        header: 'min-h-72 h-72 sm:h-136 sm:min-h-136',
      }}
      header={<OrderListHeader onSearch={handleSearch} />}
      content={
        <>
          {isLoading && <Spinner />}
          <DataTable
            hover
            page={page}
            stickyHeader
            sortBy={sortBy}
            sortDir={sortDir}
            perPage={perPage}
            isLoading={isFetching}
            count={resolvedData?.count}
            onRowClick={handleNavigateToOrder}
            paginationClasses={{ toolbar: 'ltr:pr-52 rtl:pl-52' }}
            {...tableInstance}
          />
          {console.log(resolvedData)}
          <Fab
            color="primary"
            className={classes.fab}
            aria-label="upload excel file"
          >
            <UploadIcon />
          </Fab>
        </>
      }
      innerScroll
    />
  );
}

export default OrderList;
