import {
  useRef,
  useMemo,
  useState,
  useEffect,
  forwardRef,
  useCallback,
} from 'react';
import {
  useTable,
  useSortBy,
  useRowSelect,
  usePagination,
  useGlobalFilter,
} from 'react-table';
import _ from '@lodash';
import PropTypes from 'prop-types';
import FuseUtils from '@fuse/utils';
import PropertyTable from './PropertyTable';
import Avatar from '@material-ui/core/Avatar';
import PropertySidebar from './PropertySidebar';
import { useFieldArray } from 'react-hook-form';
import Checkbox from '@material-ui/core/Checkbox';
import DeleteIcon from '@material-ui/icons/Delete';
import IconButton from '@material-ui/core/IconButton';
import Typography from '@material-ui/core/Typography';
import PropertyFormDialog from './PropertyFormDialog';
import useProperties from '../../hooks/useProperties';
import usePropertyLabels from '../../hooks/usePropertyLabels';

const IndeterminateCheckbox = forwardRef(({ indeterminate, ...rest }, ref) => {
  const defaultRef = useRef();
  const resolvedRef = ref || defaultRef;

  useEffect(() => {
    resolvedRef.current.indeterminate = indeterminate;
  }, [resolvedRef, indeterminate]);

  return <Checkbox ref={resolvedRef} indeterminate={indeterminate} {...rest} />;
});

const OrderViewProperty = ({ t, control, setValue, theme }) => {
  const [isDialogOpen, setIsDialogOpen] = useState(false);
  const [isEditDialog, setIsEditDialog] = useState(false);
  const [selectedProperty, setSelectedProperty] = useState(null);

  const { data: allProperyLabels = [] } = usePropertyLabels();
  const { data: allProperties = [] } = useProperties();

  const { append, remove, fields: properties } = useFieldArray({
    control,
    keyName: 'index',
    name: 'properties',
  });

  const handleAddProperty = (data) => {
    append({ ...data, id: FuseUtils.generateGUID() });
  };

  const handleEditProperty = ({ index, ...data }) => {
    setValue(
      'properties',
      properties.map((item) => {
        const { index, ...properyObject } = item;
        return data.id === properyObject.id ? data : properyObject;
      }),
      { shouldDirty: true }
    );
  };

  const handleGroupDelete = useCallback(
    (ids) => {
      setValue(
        'properties',
        properties.filter((property) => !ids.includes(property.id)),
        { shouldDirty: true }
      );
    },
    [properties, setValue]
  );

  const columns = useMemo(
    () => [
      {
        Header: ({ selectedFlatRows }) => {
          const selectedRowIds = selectedFlatRows.map((row) => row.original.id);

          return (
            selectedFlatRows.length > 0 && (
              <IconButton
                size="small"
                className="pr-16"
                onClick={(ev) => {
                  ev.stopPropagation();
                  handleGroupDelete(selectedRowIds);
                }}
              >
                <DeleteIcon color="error" />
              </IconButton>
            )
          );
        },
        accessor: 'PropertyIcon',
        Cell: ({ row }) => {
          return (
            <Avatar
              className="mx-8 capitalize"
              src={row.original.PropertyIcon}
              alt={row.original.property?.title}
            >
              {row.original.property?.title?.[0]}
            </Avatar>
          );
        },
        className: 'justify-center',
        width: 64,
        sortable: false,
      },
      {
        Header: t('PROPERTY'),
        accessor: 'PropertyName',
        sortable: true,
      },
      {
        Header: t('PRICE'),
        accessor: 'PropertyPrice',
        sortable: true,
      },
      {
        Header: t('DESCRIPTION'),
        accessor: 'PropertyDescription',
        Cell: ({ row }) => {
          const propertyDescription = row.original.PropertyDescription.replace(
            /<(?:.|\n)*?>/gm,
            ' '
          );
          return (
            <Typography
              className="truncate"
              color="textSecondary"
              title={propertyDescription}
            >
              {_.truncate(propertyDescription, { length: 40 })}
            </Typography>
          );
        },
        sortable: true,
      },
      {
        id: 'action',
        width: 128,
        sortable: false,
        Cell: ({ row }) => (
          <div className="flex items-center">
            <IconButton
              onClick={(ev) => {
                ev.stopPropagation();
                remove(row.index);
              }}
            >
              <DeleteIcon />
            </IconButton>
          </div>
        ),
      },
    ],
    [handleGroupDelete, remove, t]
  );

  const handleFilterData = useCallback((rows, columnIds, globalFilterValue) => {
    if (!globalFilterValue) {
      return rows;
    }

    return rows.filter(
      (row) => row.original?.propertyLabel?.id === globalFilterValue
    );
  }, []);

  const tableInstance = useTable(
    {
      columns,
      data: properties,
      autoResetPage: true,
      globalFilter: handleFilterData,
    },
    useGlobalFilter,
    useSortBy,
    usePagination,
    useRowSelect,
    (hooks) => {
      hooks.allColumns.push((_columns) => [
        // Let's make a column for selection
        {
          id: 'selection',
          sortable: false,
          Header: ({ getToggleAllRowsSelectedProps }) => (
            <div>
              <IndeterminateCheckbox {...getToggleAllRowsSelectedProps()} />
            </div>
          ),
          // The cell can use the individual row's getToggleRowSelectedProps method
          // to the render a checkbox
          Cell: ({ row }) => (
            <div>
              <IndeterminateCheckbox
                {...row.getToggleRowSelectedProps()}
                onClick={(ev) => ev.stopPropagation()}
              />
            </div>
          ),
        },
        ..._columns,
      ]);
    }
  );

  const {
    setGlobalFilter,
    state: { globalFilter = '' },
  } = tableInstance;

  const handleOpenAddDialog = () => {
    setIsDialogOpen(true);
    setIsEditDialog(false);
  };

  const handleOpenEditDialog = (row) => {
    setSelectedProperty(row.original);
    setIsEditDialog(true);
    setIsDialogOpen(true);
  };

  const handleCloseDialog = () => {
    setIsDialogOpen(false);
    setIsEditDialog(false);
    setSelectedProperty(null);
  };

  return (
    <div className="flex h-full">
      <PropertySidebar
        t={t}
        properties={properties}
        selectedLabel={globalFilter}
        allProperyLabels={allProperyLabels}
        onOpenAddDialog={handleOpenAddDialog}
        onSelectedLabelChange={setGlobalFilter}
      />

      <PropertyTable
        t={t}
        data={properties}
        tableInstance={tableInstance}
        onOpenEditDialog={handleOpenEditDialog}
      />

      {isDialogOpen && (
        <PropertyFormDialog
          theme={theme}
          edit={isEditDialog}
          isOpen={isDialogOpen}
          properties={allProperties}
          onClose={handleCloseDialog}
          formState={selectedProperty}
          properyLabels={allProperyLabels}
          onSubmit={isEditDialog ? handleEditProperty : handleAddProperty}
        />
      )}
    </div>
  );
};

OrderViewProperty.propTypes = {
  t: PropTypes.func.isRequired,
  theme: PropTypes.object.isRequired,
  setValue: PropTypes.func.isRequired,
  control: PropTypes.object.isRequired,
};

export default OrderViewProperty;
