import { useEffect, useState } from 'react';
import * as yup from 'yup';
import Tab from '@material-ui/core/Tab';
import { Link } from 'react-router-dom';
import { useParams } from 'react-router';
import Tabs from '@material-ui/core/Tabs';
import PostMedia from './PostMedia';
import Icon from '@material-ui/core/Icon';
import { useDispatch } from 'react-redux';
import { useForm } from 'react-hook-form';
import usePost from '../hooks/usePost';
import Spinner from 'app/components/Spinner';
import PostPricing from './PostPricing';
import { useUpdateEffect } from '@fuse/hooks';
import { useHistory } from 'react-router-dom';
import { useTranslation } from 'react-i18next';
import PostProperty from './PostProperty';
import FuseAnimate from '@fuse/core/FuseAnimate';
import PostBasicInfo from './PostBasicInfo';
import { useTheme } from '@material-ui/core/styles';
import useSavePost from '../hooks/useSavePost';
import Typography from '@material-ui/core/Typography';
import { yupResolver } from '@hookform/resolvers/yup';
import FusePageCarded from '@fuse/core/FusePageCarded';
import LoadingButton from 'app/components/LoadingButton';
import { showMessage } from 'app/store/fuse/messageSlice';

const validationSchema = yup.object().shape({
  name: yup.string().required(),
});

const defaultFormState = {
  wish: 0,
  like: 0,
  name: '',
  Code: '',
  info: '',
  tags: [],
  images: [],
  taxRate: 0,
  discount: 0,
  quantity: 0,
  master: null,
  cluster: null,
  publish: false,
  properties: [],
  description: '',
  priceTaxExcl: 0,
  priceTaxIncl: 0,
  courselevel: null,
  categoriesObject: [],
  featuredImageId: null,
};

const Post = () => {
  const [tabValue, setTabValue] = useState(0);
  const [featuredImage, setFeaturedImage] = useState(null);

  const theme = useTheme();
  const { t } = useTranslation('postlist');

  let { postId } = useParams();
  const history = useHistory();

  const isEdit = postId !== 'new';

  const { data, isLoading, remove } = usePost({
    postId,
    enabled: isEdit,
  });

  const dispatch = useDispatch();

  const {
    reset,
    watch,
    errors,
    control,
    register,
    setValue,
    getValues,
    handleSubmit,
    formState: { isSubmitting },
  } = useForm({
    shouldUnregister: false,
    resolver: yupResolver(validationSchema),
    defaultValues: data ? data : defaultFormState,
  });

  // console.log({ values: getValues() });

  useEffect(() => {
    console.log('reset');
    if (data) {
      reset(data);
    } else {
      reset(defaultFormState);
    }
  }, [reset, data]);

  useUpdateEffect(() => {
    if (postId === 'new') {
      remove();
      setTabValue(0);
    }
  }, [postId]);

  useEffect(() => {
    const { images = [], featuredImageId } = data || {};

    if (isEdit && featuredImageId && images.length > 0) {
      const featuredImage = images.find(
        (image) => image.id === featuredImageId
      );

      setFeaturedImage(featuredImage ? featuredImage.url : null);
    }
  }, [isEdit, data]);

  const [savePost] = useSavePost();

  const handleEdit = (data) => {
    const { courselevel, tags, ...restOfData } = data;

    if (courselevel) {
      restOfData.courselevel = courselevel.name;
    }

    restOfData.tags = tags.map((tag) => tag.value);

    return savePost({ data: restOfData, isEdit }).then(() => {
      dispatch(showMessage({ message: 'Post Saved' }));
      if (!isEdit) {
        history.push('/apps/e-laundry/postlist');
      }
    });
  };

  const handleChangeTab = (event, value) => {
    setTabValue(value);
  };

  if (isLoading) {
    return <Spinner />;
  }

  if (isEdit && !data) {
    return null;
  }

  return (
    <form className="h-full" onSubmit={handleSubmit(handleEdit)} noValidate>
      <FusePageCarded
        classes={{
          toolbar: 'p-0',
          header: 'min-h-72 h-72 sm:h-136 sm:min-h-136',
        }}
        header={
          <div className="flex flex-1 w-full items-center justify-between">
            <div className="flex flex-col items-start max-w-full">
              <FuseAnimate animation="transition.slideRightIn" delay={300}>
                <Typography
                  role="button"
                  color="inherit"
                  component={Link}
                  to="/apps/e-laundry/postlist"
                  className="normal-case flex items-center sm:mb-12"
                >
                  <Icon className="text-20">
                    {theme.direction === 'ltr' ? 'arrow_back' : 'arrow_forward'}
                  </Icon>
                  <span className="mx-4">{t('PRODUCTS')}</span>
                </Typography>
              </FuseAnimate>

              <div className="flex items-center max-w-full">
                <FuseAnimate animation="transition.expandIn" delay={300}>
                  {/* {isEdit &&
                  data?.images?.length > 0 &&
                  data?.featuredImageId ? (
                    <img
                      src={data?.images?.[0].url}
                      alt={data?.name || 'Post'}
                      className="w-32 sm:w-48 rounded"
                    />
                  )  */}
                  {isEdit && featuredImage ? (
                    <img
                      src={featuredImage}
                      alt={data?.name || 'Post'}
                      className="w-32 sm:w-48 rounded"
                    />
                  ) : (
                    <img
                      alt={'New Post'}
                      className="w-32 sm:w-48 rounded"
                      src={`${process.env.PUBLIC_URL}/assets/images/ecommerce/post-image-placeholder.png`}
                    />
                  )}
                </FuseAnimate>
                <div className="flex flex-col min-w-0 mx-8 sm:mc-16">
                  <FuseAnimate animation="transition.slideLeftIn" delay={300}>
                    <Typography className="text-16 sm:text-20 truncate">
                      {isEdit && data ? data.name : t('NEW_PRODUCT')}
                    </Typography>
                  </FuseAnimate>
                </div>
              </div>
            </div>
            <LoadingButton
              type="submit"
              color="secondary"
              variant="contained"
              isLoading={isSubmitting}
              className="whitespace-no-wrap normal-case"
            >
              {t('SAVE')}
            </LoadingButton>
          </div>
        }
        contentToolbar={
          <Tabs
            value={tabValue}
            textColor="primary"
            variant="scrollable"
            scrollButtons="auto"
            indicatorColor="primary"
            onChange={handleChangeTab}
            classes={{ root: 'w-full h-64' }}
          >
            <Tab className="h-64 normal-case" label={t('BASIC_INFO')} />
            <Tab className="h-64 normal-case" label={t('MEDIA')} />
            <Tab className="h-64 normal-case" label={t('PRICING')} />
            <Tab className="h-64 normal-case" label={t('PROPERTY')} />
          </Tabs>
        }
        content={
          <div className="p-16 sm:p-24 max-w-2xl">
            {tabValue === 0 && (
              <PostBasicInfo
                t={t}
                theme={theme}
                errors={errors}
                control={control}
                register={register}
                getValues={getValues}
                setValue={setValue}
              />
            )}
            {tabValue === 1 && (
              <PostMedia
                t={t}
                watch={watch}
                control={control}
                setValue={setValue}
              />
            )}
            {tabValue === 2 && (
              <PostPricing
                t={t}
                control={control}
                register={register}
                setValue={setValue}
                getValues={getValues}
              />
            )}
            {tabValue === 3 && (
              <PostProperty
                t={t}
                theme={theme}
                control={control}
                setValue={setValue}
              />
            )}
          </div>
        }
        innerScroll
      />
    </form>
  );
};

Post.propTypes = {};

export default Post;
