import * as yup from 'yup';
import PropTypes from 'prop-types';
import Icon from '@material-ui/core/Icon';
import { getImageUrl } from 'app/utils/url';
import AppBar from '@material-ui/core/AppBar';
import Dialog from '@material-ui/core/Dialog';
import ImageBox from 'app/components/ImageBox';
import Divider from '@material-ui/core/Divider';
import Toolbar from '@material-ui/core/Toolbar';
import Checkbox from '@material-ui/core/Checkbox';
import TextField from '@material-ui/core/TextField';
import { amber, red } from '@material-ui/core/colors';
import IconButton from '@material-ui/core/IconButton';
import Typography from '@material-ui/core/Typography';
import { useForm, Controller } from 'react-hook-form';
import { yupResolver } from '@hookform/resolvers/yup';
import { makeStyles } from '@material-ui/core/styles';
import LoadingButton from 'app/components/LoadingButton';
import DialogActions from '@material-ui/core/DialogActions';
import DialogContent from '@material-ui/core/DialogContent';
import DialogCloseButton from 'app/components/DialogCloseButton';
import FormControlLabel from '@material-ui/core/FormControlLabel';
import useDeletePropertyLabel from '../hooks/useDeletePropertyLabel';

const useStyles = makeStyles((theme) => ({
  dialogContent: {
    padding: '0 !important',
  },
}));

const validationSchema = yup.object().shape({
  title: yup.string().required(),
});

const defaultFormState = {
  title: '',
  handle: '',
  iconSrc: '',
  isColor: false,
  starred: false,
  description: '',
  important: false,
};

function PropertyLabelFormDialog({ t, edit, data, onClose, onSubmit }) {
  const classes = useStyles();

  const {
    watch,
    reset,
    errors,
    control,
    register,
    setValue,
    getValues,
    handleSubmit,
    formState: { isSubmitting },
  } = useForm({
    shouldUnregister: false,
    resolver: yupResolver(validationSchema),
    defaultValues: edit ? data : defaultFormState,
  });

  const [
    deletePropertyLabel,
    { isLoading: isDeleting },
  ] = useDeletePropertyLabel();

  const handleSubmitForm = (data) => {
    return onSubmit(data).then((response) => {
      if (edit) {
        reset(response);
      } else {
        onClose();
      }
    });
  };

  const handleToggleImportant = () => {
    const isImportant = getValues('important');
    setValue('important', !isImportant);
  };

  const handleToggleStarred = () => {
    const isStarred = getValues('starred');
    setValue('starred', !isStarred);
  };

  const handleUploadChange = (e, fieldName) => {
    const file = e.target.files[0];

    setValue(fieldName + 'Src', URL.createObjectURL(file));
    setValue(fieldName + 'File', file);

    e.target.value = null;
  };

  const handleDeleteImage = (fieldName) => {
    setValue(fieldName + 'Src', '');
    setValue(fieldName + 'File', '');
  };

  const handleDeleteProperty = () => {
    deletePropertyLabel(data?.id).then(() => {
      onClose();
    });
  };

  const formValues = watch([
    'starred',
    'iconSrc',
    'imageSrc',
    'iconFile',
    'important',
    'imageFile',
  ]);

  return (
    <Dialog
      fullWidth
      open={true}
      maxWidth="sm"
      onClose={onClose}
      disableBackdropClick
      classes={{ paper: 'rounded-8' }}
    >
      <AppBar position="static" elevation={1}>
        <Toolbar className="flex w-full">
          <Typography variant="subtitle1" color="inherit">
            {edit ? t('EDIT_PROPERTY_LABEL') : t('NEW_PROPERTY_LABEL')}
          </Typography>
        </Toolbar>
        <DialogCloseButton onClick={onClose} />
      </AppBar>

      <form
        noValidate
        onSubmit={handleSubmit(handleSubmitForm)}
        className="flex flex-col md:overflow-hidden"
      >
        <DialogContent classes={{ root: classes.dialogContent }}>
          <div className="mb-16">
            <div className="flex items-center justify-between p-12">
              <div className="flex">
                <FormControlLabel
                  control={
                    <Controller
                      name="isColor"
                      render={(props) => (
                        <Checkbox
                          checked={props.value}
                          onChange={(e) => props.onChange(e.target.checked)}
                        />
                      )}
                      control={control}
                    />
                  }
                  label={t('IS_COLOR')}
                />
              </div>

              <div
                className="flex items-center justify-start"
                aria-label="Toggle star"
              >
                <IconButton onClick={handleToggleImportant}>
                  {formValues.important ? (
                    <Icon style={{ color: red[500] }}>error</Icon>
                  ) : (
                    <Icon>error_outline</Icon>
                  )}
                </IconButton>

                <IconButton onClick={handleToggleStarred}>
                  {formValues.starred ? (
                    <Icon style={{ color: amber[500] }}>star</Icon>
                  ) : (
                    <Icon>star_outline</Icon>
                  )}
                </IconButton>
              </div>
            </div>
            <Divider className="mx-24" />
          </div>

          <div className="px-16 sm:px-24">
            <TextField
              required
              fullWidth
              autoFocus
              type="text"
              name="title"
              className="mb-24"
              label={t('TITLE')}
              variant="outlined"
              inputRef={register}
              error={!!errors?.title}
              helperText={errors?.title?.message}
            />
            {/* TODO: add types */}

            <TextField
              rows="3"
              fullWidth
              multiline
              type="text"
              name="notes"
              className="mb-16"
              variant="outlined"
              inputRef={register}
              label={t('DESCRIPTION')}
            />

            <div className="flex items-center flex-wrap -mx-16 mb-32">
              <ImageBox
                t={t}
                name="icon"
                accept="image/png"
                onDelete={() => handleDeleteImage('icon')}
                onChange={(e) => handleUploadChange(e, 'icon')}
                imageSource={
                  formValues.iconFile
                    ? formValues.iconSrc
                    : getImageUrl(formValues.iconSrc)
                }
              />
            </div>
          </div>
        </DialogContent>

        <DialogActions className="justify-between p-8">
          <div className="px-16">
            <LoadingButton
              type="submit"
              color="primary"
              variant="contained"
              isLoading={isSubmitting || isDeleting}
              className="whitespace-no-wrap normal-case"
            >
              {t('SAVE')}
            </LoadingButton>
          </div>
          {edit && (
            <IconButton
              disabled={isDeleting}
              className="min-w-auto"
              onClick={handleDeleteProperty}
            >
              <Icon>delete</Icon>
            </IconButton>
          )}
        </DialogActions>
      </form>
    </Dialog>
  );
}

PropertyLabelFormDialog.propTypes = {
  t: PropTypes.func.isRequired,
};

export default PropertyLabelFormDialog;
