import { useEffect, useReducer } from 'react';
import Marker from './Marker';
import SearchBox from './SearchBox';
import Tab from '@material-ui/core/Tab';
import { useParams } from 'react-router';
import Tabs from '@material-ui/core/Tabs';
import Spinner from 'app/components/Spinner';
import GoogleMapReact from 'google-map-react';
import { useTranslation } from 'react-i18next';
import GodsViewMasters from './GodsViewMasters';
import FuseAnimate from '@fuse/core/FuseAnimate';
import MasterInfoDialog from './MasterInfoDialog';
import useMasterList from '../hooks/useMasterList';
import Typography from '@material-ui/core/Typography';
import FusePageCarded from '@fuse/core/FusePageCarded';

const defaultMapSettings = {
  center: {
    lat: 59.95,
    lng: 30.33,
  },
  zoom: 11,
};

const fitBounds = (mapInstance, mapApi, places) => {
  if (!mapInstance || !mapApi) {
    return;
  }

  const bounds = new mapApi.LatLngBounds();

  places.forEach((place) => {
    bounds.extend(
      new mapApi.LatLng(place.masterLocation[0], place.masterLocation[1])
    );
  });

  mapInstance?.fitBounds(bounds);
};

function filterData(entities, searchText, filter) {
  if (!entities?.length) {
    return [];
  }

  let filteredData = entities;

  if (searchText) {
    filteredData = filterBySearchQuery(filteredData, searchText);
  }

  if (filter) {
    filteredData = filterByStatus(filteredData, filter);
  }

  return filteredData;
}

function filterBySearchQuery(entities, searchText) {
  if (searchText.length === 0) {
    return entities;
  }

  return entities.filter((item) => item.masterName.includes(searchText));
}

function filterByStatus(entities, filter) {
  if (filter === 'all') {
    return entities;
  }

  return entities.filter((item) => {
    if (item.status === filter) {
      return true;
    }

    return false;

    // if (filter === 'free' && item.trip === 0 && item.status === 'free') {
    // if (filter === 'free' && item.status === filter) {
    //   return true;
    // }

    // if (filter === 'busy' && item.status === 'busy') {
    //   return true;
    // }

    // if (filter === 'deactive' && item.status === 'deactive') {
    //   return true;
    // }
  });
}

const initialState = {
  mapApi: null,
  filter: 'all',
  searchText: '',
  mapInstance: null,
  filteredData: null,
  hoveredMarkerId: null,
  isInfoDialogOpen: false,
  selectedMasterInfo: null,
};

function reducer(state, action) {
  switch (action.type) {
    case 'SET_FILTERED_DATA':
      const { data, searchText, filter } = action;
      let filteredData = [];
      if (data?.length) {
        filteredData = filterData(data, searchText, filter);
      }
      return { ...state, filteredData };

    case 'SET_FILTER':
      return { ...state, filter: action.filter };

    case 'SET_SEARCH_TEXT':
      return { ...state, searchText: action.searchText };

    case 'OPEN_DIALOG':
      return {
        ...state,
        isInfoDialogOpen: true,
        selectedMasterInfo: action.selectedMasterInfo,
      };

    case 'CLOSE_DIALOG':
      return { ...state, isInfoDialogOpen: false, selectedMasterInfo: null };

    case 'SET_HOVERED_MARKER_ID':
      return { ...state, hoveredMarkerId: action.id };

    case 'UNSET_HOVERED_MARKER_ID':
      return {
        ...state,
        hoveredMarkerId: state.id === action.itemId ? null : state.id,
      };

    case 'SET_MAP_INSTANCES':
      return {
        ...state,
        mapApi: action.maps,
        mapInstance: action.map,
      };

    default:
      throw new Error();
  }
}

const GodsView = (props) => {
  const [state, setState] = useReducer(reducer, initialState);

  const {
    filter,
    mapApi,
    searchText,
    mapInstance,
    hoveredMarkerId,
    filteredData,
    isInfoDialogOpen,
    selectedMasterInfo,
  } = state;

  const { t, i18n } = useTranslation('reports');

  let { postId } = useParams();

  const { data, isLoading } = useMasterList({ postId });

  useEffect(() => {
    console.log('useEffect', { searchText, filter, data });
    setState({ type: 'SET_FILTERED_DATA', data, searchText, filter });
  }, [searchText, filter, data]);

  useEffect(() => {
    fitBounds(mapInstance, mapApi, filteredData);
  }, [mapInstance, mapApi, filteredData]);

  const handleChangeFilter = (event, filter) => {
    setState({ type: 'SET_FILTER', filter });
  };

  const handleOpenInfoDialog = (item) => {
    setState({ type: 'OPEN_DIALOG', selectedMasterInfo: item });
  };

  const handleCloseInfoDialog = () => {
    setState({ type: 'CLOSE_DIALOG' });
  };

  const handleHoverMarker = (id) => {
    setState({ type: 'SET_HOVERED_MARKER_ID', id });
  };

  const handleLeaveMarker = (itemId) => {
    setState({ type: 'UNSET_HOVERED_MARKER_ID', itemId });
  };

  const handleSearchTextChange = (e) => {
    setState({ type: 'SET_SEARCH_TEXT', searchText: e.target.value });
  };

  const handleShowItemOnTheMap = (item) => {
    if (!mapInstance || !mapApi) {
      return;
    }

    const [lat, lng] = item.masterLocation;
    const latLng = new mapApi.LatLng(lat, lng);

    handleHoverMarker(item.id);

    mapInstance.setZoom(17);
    mapInstance.panTo(latLng);
  };

  if (isLoading) {
    return <Spinner />;
  }

  if (!filteredData) {
    return null;
  }

  const handleApiLoaded = ({ map, maps }) => {
    setState({ type: 'SET_MAP_INSTANCES', map, maps });

    fitBounds(map, maps, filteredData);
  };

  console.log({ filteredData });

  return (
    <FusePageCarded
      classes={{
        root: 'h-full',
        content: 'flex p-16 sm:p-24',
        toolbar: 'p-0',
        contentCard: 'overflow-hidden',
        header: 'min-h-72 h-72 sm:h-136 sm:min-h-136',
      }}
      header={
        <div className="flex flex-1 w-full items-center justify-between max-w-full">
          <div className="flex flex-col min-w-0 mx-8 sm:mc-16">
            <FuseAnimate animation="transition.slideLeftIn" delay={300}>
              <Typography className="text-16 sm:text-20 truncate">
                {t('MASTER')}
              </Typography>
            </FuseAnimate>
          </div>
        </div>
      }
      contentToolbar={
        <>
          <Tabs
            value={filter}
            textColor="primary"
            variant="scrollable"
            scrollButtons="auto"
            indicatorColor="primary"
            onChange={handleChangeFilter}
            classes={{ root: 'w-full h-64' }}
          >
            <Tab
              value="all"
              className="h-64 normal-case"
              label={t('GODEVIEW_ALL_MASTER')}
            />
            <Tab
              value="busy"
              className="h-64 normal-case"
              label={t('GODEVIEW_BUSY_MASTER')}
            />
            <Tab
              value="free"
              className="h-64 normal-case"
              label={t('GODEVIEW_FREE_MASTER')}
            />
            <Tab
              value="deactive"
              className="h-64 normal-case"
              label={t('GODEVIEW_INACTIVE_MASTER')}
            />
          </Tabs>
        </>
      }
      content={
        <div className="h-full w-full relative">
          {mapApi && <SearchBox mapInstance={mapInstance} mapApi={mapApi} />}

          <GoogleMapReact
            bootstrapURLKeys={{
              key: process.env.REACT_APP_GOOGLE_MAP_KEY,
              libraries: ['places'],
            }}
            yesIWantToUseGoogleMapApiInternals
            defaultZoom={defaultMapSettings.zoom}
            defaultCenter={defaultMapSettings.center}
            onGoogleApiLoaded={handleApiLoaded}
          >
            {filteredData?.map((item) => (
              <Marker
                key={item.id}
                data={item}
                currentLang={i18n.language}
                lat={item.masterLocation[0]}
                lng={item.masterLocation[1]}
                isHovered={hoveredMarkerId === item.id}
                onClick={() => handleOpenInfoDialog(item)}
                onHover={() => handleHoverMarker(item.id)}
                onLeave={() => handleLeaveMarker(item.id)}
              />
            ))}
          </GoogleMapReact>
          {isInfoDialogOpen && (
            <MasterInfoDialog
              isOpen={isInfoDialogOpen}
              onClose={handleCloseInfoDialog}
              selectedItem={selectedMasterInfo}
            />
          )}
        </div>
      }
      leftSidebarHeader={<></>}
      leftSidebarContent={
        <GodsViewMasters
          isLoading={isLoading}
          searchText={searchText}
          masterList={filteredData}
          onClick={handleShowItemOnTheMap}
          onSearchTextChange={handleSearchTextChange}
        />
      }
    />
  );
};

GodsView.propTypes = {};

export default GodsView;
