import { useState, useRef } from 'react';
import clsx from 'clsx';
import * as yup from 'yup';
import { Link } from 'react-router-dom';
import Card from '@material-ui/core/Card';
import Icon from '@material-ui/core/Icon';
import { useForm } from 'react-hook-form';
import Button from '@material-ui/core/Button';
import { useTranslation } from 'react-i18next';
import FuseAnimate from '@fuse/core/FuseAnimate';
import TextField from '@material-ui/core/TextField';
import { yupResolver } from '@hookform/resolvers/yup';
import { makeStyles } from '@material-ui/core/styles';
import Typography from '@material-ui/core/Typography';
import IconButton from '@material-ui/core/IconButton';
import { useDispatch, useSelector } from 'react-redux';
import CardContent from '@material-ui/core/CardContent';
import { submitLogin } from 'app/auth/store/loginSlice';
import InputAdornment from '@material-ui/core/InputAdornment';
import { darken } from '@material-ui/core/styles/colorManipulator';
import { green } from '@material-ui/core/colors';
import CircularProgress from '@material-ui/core/CircularProgress';
import FormControl from "@material-ui/core/FormControl";
import FormControlLabel from "@material-ui/core/FormControlLabel";
import Checkbox from "@material-ui/core/Checkbox"; 


const useStyles = makeStyles((theme) => ({
  root: {
    background:"white",
    color: theme.palette.primary.contrastText,
  },
  leftSection: {},
  rightSection: {
    background: 'url("/assets/images/logos/lofinSplash.jpg") !important  ',
    backgroundSize:"cover",
    BackgroundRepeat:'no-repeat',
    with:'100%', height:'100%',

  },
  buttonProgress: {
    color: green[500],
    position: 'absolute',
    top: '50%',
    left: '50%',
    marginTop: -8,
    marginLeft: -12,
  },
  wrapper: {

    position: 'relative',
  },
  loginstyle: {
    '& > *': {
      margin: theme.spacing(2),
    },
  },

}));

const validationSchema = yup.object().shape({
  email: yup.string().required(),
  password: yup.string().required(),
});

function Login() {
  const { t } = useTranslation('loginPage');

  const [showPassword, setShowPassword] = useState(false);

  const classes = useStyles();

  const [loading, setLoading] = useState(false);
  const [success, setSuccess] = useState(false);
  const timer = useRef();

  const dispatch = useDispatch();
  const login = useSelector(({ auth }) => auth.login);

  const { register, errors, handleSubmit } = useForm({
    defaultValues: {
      email: '',
      password: '',
    },
    resolver: yupResolver(validationSchema),
  });

  const handleSubmitLogin = (data) => {
    dispatch(submitLogin(data));
  };

  const buttonClassname = clsx({
    [classes.buttonSuccess]: success,
  });
  const handleButtonClick = () => {
    if (!loading) {
      setSuccess(false);
      setLoading(true);
      timer.current = window.setTimeout(() => {
        setSuccess(true);
        setLoading(false);
      }, 2000);
    }
  };

  return (
    <div
      className={clsx(
        classes.root,
        'flex flex-col flex-auto items-center justify-center flex-shrink-0 p-0 md:p-0'
      )}
    >
      <FuseAnimate animation="transition.expandIn">
        <div className="flex w-full h-full  md:max-w-full  shadow-2xl overflow-hidden">
          <Card
            className={clsx(
              classes.leftSection,
              'flex flex-col w-full max-w-sm items-center pb-32 justify-center'
            )}
            square
            elevation={0}
          >
            <CardContent className="flex flex-col items-center justify-center w-full p-0 pt-16 max-w-320">
              <FuseAnimate delay={300}>
                <div className="mb-64">
                  <FuseAnimate animation="transition.expandIn">
                    <img
                      alt="logo"
                      className="mb-4 mt-4"
                      src={`${process.env.PUBLIC_URL}/assets/images/logos/sbsmslogo.svg`}
                    />
                  </FuseAnimate>
                </div>
              </FuseAnimate>

              <div className="w-full">
                <form
                  noValidate
                  name="loginForm"
                  onSubmit={handleSubmit(handleSubmitLogin)}
                  className="flex flex-col justify-center w-full"
                >
                  <TextField
                    required
                    type="text"
                    name="email"
                    className="mb-16"
                    variant="outlined"
                    inputRef={register}
                    error={!!errors?.email}
                    label="نام کاربری/ایمیل" 
                    margin="dense"
                    helperText={errors?.email?.message}
                    
                    InputProps={{
                      startAdornment: <InputAdornment position="start"> </InputAdornment>,
                      endAdornment: (                        
                        <InputAdornment position="end">
                          <Icon className="text-20" color="action">
                            email
                          </Icon>
                        </InputAdornment>
                      ),
                    }}
                  />

                  <TextField
                    required
                    type="password"
                    name="password"
                    className="mb-16"
                    label="کلمه عبور" 
                    margin="dense"
                    variant="outlined"
                    inputRef={register}
                    error={!!errors?.password}
                    helperText={errors?.password?.message}
                    InputProps={{
                      className: 'pr-2',
                      type: showPassword ? 'text' : 'password',
                      startAdornment: <InputAdornment position="start"> </InputAdornment>,
                      endAdornment: (
                        <InputAdornment position="end">
                          <IconButton
                            onClick={() => setShowPassword(!showPassword)}
                            size="small"
                          >
                            <Icon className="text-20" color="action">
                              {showPassword ? 'visibility' : 'visibility_off'}
                            </Icon>
                          </IconButton>
                        </InputAdornment>
                      ),
                    }}
                  />

                  <div className="flex items-center justify-between">
                    <FormControl>
                      <FormControlLabel
                        control={<Checkbox name="remember" inputRef={register} />}
                        label={t('REMEMBER_ME')}
                      />
                    </FormControl>
 
                  </div>

                  <div className={classes.wrapper}>
                    <Button
                      type="submit"
                      variant="contained"
                      color="primary"
                      className={clsx('w-full mx-auto mt-16 normal-case', buttonClassname)}
                      //className={buttonClassname}
                      // className="w-full mx-auto mt-16 normal-case"
                      onClick={handleButtonClick}
                      aria-label={t('LOGIN')}
                      disabled={loading}
                    >
                      {t('LOGIN')}
                    </Button>
                    {loading && <CircularProgress size={24} className={classes.buttonProgress} />}
                  </div>

                </form>
              </div>
            </CardContent>


            <div className={classes.loginstyle}>
              <Link to="/register" >
                <Button
                  color="action"
                  className=""
                  startIcon={<Icon>people</Icon>}
                >
                  ثبت نام پنل رایگان
                </Button>
              </Link>
              <Link to="/forgetpass" >
                <Button
                  color="primary"
                  startIcon={<Icon>vpn_key</Icon>}
                >
                  بازیابی رمز ورود
                </Button>
              </Link>
            </div>
            
          </Card>

          <div
            className={clsx(
              classes.rightSection,
              'hidden md:flex flex-1 items-center justify-center p-64'
            )}

          >
            <div className="max-w-320 ">


              <FuseAnimate animation="transition.slideUpIn" delay={400}>
                <Typography
                  variant="h3"
                  color="inherit"
                  className="leading-tight"
                >
                  
                </Typography>
              </FuseAnimate>

              <FuseAnimate delay={500}>
                <Typography
                  variant="subtitle1"
                  color="inherit"
                  className="mt-32"
                >
                  
                </Typography>
              </FuseAnimate>
            </div>
          </div>
        </div>
      </FuseAnimate>
    </div>
  );
}

export default Login;
