import { useEffect, useMemo, useState } from 'react';
import qs from 'qs';
import PropTypes from 'prop-types';
import Paper from '@material-ui/core/Paper';
import Spinner from 'app/components/Spinner';
import DataTable from 'app/components/DataTable';
import useTransactions from '../../hooks/useTransactions';
import { useLocation, useHistory } from 'react-router-dom';
import useDataTable from 'app/components/DataTable/useDataTable';

const OrderViewTransactions = ({ t, postId }) => {
  const history = useHistory();
  const { search: locationSearch } = useLocation();

  const urlParams = qs.parse(locationSearch, {
    ignoreQueryPrefix: true,
  });

  const [queryPerPage, setQueryPerPage] = useState(
    parseInt(urlParams.perPage) || 10
  );
  const [queryPage, setQueryPage] = useState(parseInt(urlParams.page) || 1);
  const [querySortBy, setQuerySortBy] = useState(urlParams.sortBy);
  const [querySortDir, setQuerySortDir] = useState(urlParams.sortDir);
  const [querySearch, setQuerySearch] = useState(urlParams.search);

  const {
    isLoading,
    isFetching,
    resolvedData = { count: 0, transactions: [], pageCount: 0 },
  } = useTransactions({
    postId,
    page: queryPage,
    search: querySearch,
    sortBy: querySortBy,
    perPage: queryPerPage,
    sortDir: querySortDir,
  });

  const columns = useMemo(
    () => [
      { header: 'AccType', accessor: 'AccType', sortable: true },
      { header: 'amount', accessor: 'amount', sortable: true },
      { header: 'gatewayTitle', accessor: 'gatewayTitle', sortable: true },
      { header: 'totalCount', accessor: 'totalCount', sortable: true },
      { header: 'AccCenter', accessor: 'AccCenter', sortable: true },
      { header: 'refrenseType', accessor: 'refrenseType', sortable: true },
      { header: 'des', accessor: 'des', sortable: true },
    ],
    [t]
  );

  const {
    page,
    sortBy,
    search,
    perPage,
    sortDir,
    handleSearch,
    appliedFilters,
    ...tableInstance
  } = useDataTable({
    columns,
    initialPage: queryPage,
    initialSortBy: querySortBy,
    initialSearch: querySearch,
    data: resolvedData?.transactions,
    initialPerPage: queryPerPage,
    initialSortDir: querySortDir,
  });

  useEffect(() => {
    setQueryPage(page);
  }, [page]);

  useEffect(() => {
    setQueryPerPage(perPage);
  }, [perPage]);

  useEffect(() => {
    setQuerySearch(search);
  }, [search]);

  useEffect(() => {
    if (sortBy !== querySortBy) {
      setQuerySortBy(sortBy);
    }

    if (sortDir !== querySortDir) {
      setQuerySortDir(sortDir);
    }
  }, [sortBy, sortDir, querySortBy, querySortDir]);

  return (
    <div className="flex w-full h-full">
      {isLoading && <Spinner />}
      <Paper className="w-full rounded-8 shadow-1">
        <DataTable
          page={page}
          stickyHeader
          sortBy={sortBy}
          sortDir={sortDir}
          perPage={perPage}
          isLoading={isFetching}
          count={resolvedData?.count}
          {...tableInstance}
        />
      </Paper>
    </div>
  );
};

OrderViewTransactions.propTypes = {
  t: PropTypes.func.isRequired,
  postId: PropTypes.string.isRequired,
};

export default OrderViewTransactions;
