import Icon from '@material-ui/core/Icon';
import Table from '@material-ui/core/Table';
import AppBar from '@material-ui/core/AppBar';
import Button from '@material-ui/core/Button';
import Dialog from '@material-ui/core/Dialog';
import useSaveRole from '../hooks/useSaveRole';
import { useTranslation } from 'react-i18next';
import Toolbar from '@material-ui/core/Toolbar';
import TableRow from '@material-ui/core/TableRow';
import Checkbox from '@material-ui/core/Checkbox';
import useCreateRole from '../hooks/useCreateRole';
import TextField from '@material-ui/core/TextField';
import TableBody from '@material-ui/core/TableBody';
import TableCell from '@material-ui/core/TableCell';
import TableHead from '@material-ui/core/TableHead';
import { useForm, Controller } from 'react-hook-form';
import IconButton from '@material-ui/core/IconButton';
import Typography from '@material-ui/core/Typography';
import DialogActions from '@material-ui/core/DialogActions';
import DialogContent from '@material-ui/core/DialogContent';
import DialogCloseButton from 'app/components/DialogCloseButton';

function RoleFormDialog({
  isOpen,
  routes,
  onClose,
  onDelete,
  edit = false,
  selectedRole,
}) {
  const { register, handleSubmit, control } = useForm();
  const { t } = useTranslation('settings');

  const [createRole] = useCreateRole();
  const [saveRole] = useSaveRole();

  const handleSubmitForm = ({ name, ...data }) => {
    const roleObject = { role: name, access: {} };
    const selectedPermissions = Object.keys(data).filter(
      (permission) => data[permission] !== false
    );

    selectedPermissions.forEach((item) => {
      const [role, permission] = item.split('___');
      if (Array.isArray(roleObject.access[role])) {
        roleObject.access[role] = [...roleObject.access[role], permission];
      } else {
        roleObject.access[role] = [permission];
      }
    });

    if (edit) {
      roleObject.id = selectedRole.id;
      saveRole(roleObject);
    } else {
      createRole(roleObject).then(() => onClose());
    }
  };

  return (
    <Dialog
      fullWidth
      maxWidth="md"
      open={isOpen}
      disableBackdropClick
      onClose={onClose}
      classes={{
        paper: 'm-24 rounded-8',
      }}
    >
      <AppBar position="static" elevation={1}>
        <Toolbar className="flex w-full">
          <Typography variant="subtitle1" color="inherit">
            {edit ? t('EDIT_ROLE') : t('ADD_ROLE')}
          </Typography>
        </Toolbar>
        <DialogCloseButton onClick={onClose} />
      </AppBar>
      <form
        noValidate
        onSubmit={handleSubmit(handleSubmitForm)}
        className="flex flex-col md:overflow-hidden"
      >
        <DialogContent classes={{ root: 'p-24' }}>
          <div className="flex flex-col">
            <TextField
              required
              autoFocus
              name="name"
              label={t('ROLE')}
              variant="outlined"
              inputRef={register}
              className="mt-8 mb-16 mx-4"
              defaultValue={edit ? selectedRole.role : ''}
            />

            <Table className="my-20">
              <TableHead>
                <TableRow>
                  <TableCell align="center">Route</TableCell>
                  <TableCell align="center">Read</TableCell>
                  <TableCell align="center">Create</TableCell>
                  <TableCell align="center">Write</TableCell>
                  <TableCell align="center">Delete</TableCell>
                </TableRow>
              </TableHead>
              <TableBody>
                {routes.map((route) => (
                  <TableRow key={route}>
                    <TableCell align="center">{route}</TableCell>
                    {['read', 'create', 'write', 'delete'].map((permission) => {
                      let defaultValue = false;

                      if (edit) {
                        const hasPermission = selectedRole?.access[
                          route
                        ]?.indexOf(permission);
                        defaultValue =
                          hasPermission !== undefined && hasPermission !== -1;
                      }

                      return (
                        <TableCell
                          align="center"
                          key={`${route}___${permission}`}
                        >
                          <Controller
                            name={`${route}___${permission}`}
                            control={control}
                            defaultValue={defaultValue}
                            render={(props) => (
                              <Checkbox
                                onChange={(e) =>
                                  props.onChange(e.target.checked)
                                }
                                checked={props.value ? true : false}
                              />
                            )}
                          />
                        </TableCell>
                      );
                    })}
                  </TableRow>
                ))}
              </TableBody>
            </Table>
          </div>
        </DialogContent>

        {!edit ? (
          <DialogActions className="justify-between p-8">
            <div className="px-16">
              <Button
                variant="contained"
                color="primary"
                type="submit"
                // disabled={!canBeSubmitted()}
              >
                {t('ADD')}
              </Button>
            </div>
          </DialogActions>
        ) : (
          <DialogActions className="justify-between p-8">
            <div className="px-16">
              <Button
                variant="contained"
                color="primary"
                type="submit"
                // disabled={!canBeSubmitted()}
              >
                {t('EDIT')}
              </Button>
            </div>
            <IconButton
              onClick={() => {
                onDelete(selectedRole?.id);
                onClose();
              }}
            >
              <Icon color="error">delete</Icon>
            </IconButton>
          </DialogActions>
        )}
      </form>
    </Dialog>
  );
}

export default RoleFormDialog;
