import { useEffect, useMemo, useState } from 'react';
import qs from 'qs';
import useRoles from '../hooks/useRoles';
import Icon from '@material-ui/core/Icon';
import useRoutes from '../hooks/useRoutes';
import Header from './RolesManagementHeader';
import RoleFormDialog from './RoleFormDialog'; 
import { useTranslation } from 'react-i18next';
import { useLocation } from 'react-router-dom';
import DataTable from 'app/components/DataTable';
import useDeleteRole from '../hooks/useDeleteRole';
import useDeleteRoles from '../hooks/useDeleteRoles';
import IconButton from '@material-ui/core/IconButton';
import FusePageCarded from '@fuse/core/FusePageCarded';
import DeleteIcon from '@material-ui/icons/DeleteOutline';
import useDataTable from 'app/components/DataTable/useDataTable';
import DataTableCheckbox from 'app/components/DataTable/DataTableCheckbox';

function RolesManagement() {
  const [isDialogOpen, setIsDialogOpen] = useState(false);
  const [isEditDialog, setIsEditDialog] = useState(false);
  const [selectedRole, setSelectedRole] = useState(null);

  const { t } = useTranslation('settings');

  const { search: locationSearch } = useLocation();

  const urlParams = qs.parse(locationSearch, {
    ignoreQueryPrefix: true,
  });

  const [queryPerPage, setQueryPerPage] = useState(
    parseInt(urlParams.perPage) || 10
  );
  const [queryPage, setQueryPage] = useState(parseInt(urlParams.page) || 1);
  const [querySortBy, setQuerySortBy] = useState(urlParams.sortBy);
  const [querySortDir, setQuerySortDir] = useState(urlParams.sortDir);
  const [querySearch, setQuerySearch] = useState(urlParams.search);

  const { data: routes } = useRoutes();

  const {
    // status,
    // isLoading,
    // isFetching,
    resolvedData = { count: 0, roles: [], pageCount: 0 },
  } = useRoles({
    page: queryPage,
    search: querySearch,
    sortBy: querySortBy,
    perPage: queryPerPage,
    sortDir: querySortDir,
  });

  const [deleteRole] = useDeleteRole();
  const [deleteRoles] = useDeleteRoles();

  const columns = useMemo(
    () => [
      {
        id: 'selection',
        header: ({
          selectedRows,
          isAllRowsSelected,
          isIndeterminateSelect,
          onToggleAllSelectedRow,
        }) => {
          const selectedRowIds = Object.keys(selectedRows);

          return (
            <div className="flex">
              <DataTableCheckbox
                className="mx-10"
                checked={isAllRowsSelected}
                indeterminate={isIndeterminateSelect}
                onChange={(e) => onToggleAllSelectedRow(e.target.checked)}
              />

              {selectedRowIds.length > 0 && (
                // <MultiSelectMenu selectedIds={selectedRowIds} />
                <div className="flex items-center justify-center h-64 mx-10">
                  <IconButton
                    onClick={(e) => {
                      e.stopPropagation();
                      deleteRoles(selectedRowIds).then(() => {
                        onToggleAllSelectedRow(false);
                      });
                    }}
                  >
                    <Icon color="error">delete</Icon>
                  </IconButton>
                </div>
              )}
            </div>
          );
        },
        accessor: ({ row, isSelected, handleToggleSelectedRow }) => (
          <div>
            <DataTableCheckbox
              className="mx-10"
              checked={isSelected}
              onChange={(e) =>
                handleToggleSelectedRow(row.id, e.target.checked)
              }
            />
          </div>
        ),
        headerClassName: 'w-136',
        disablePadding: true,
      },
      {
        sortable: true,
        accessor: 'role',
        header: t('ROLE'),
        disablePadding: true,
        className: 'p-4 md:p-16',
        headerClassName: 'p-4 md:p-16',
      },
      {
        sortable: true,
        accessor: 'role',
        id: 'description',
        disablePadding: true,
        header: t('DESCRIPTION'),
        className: 'p-4 md:p-16',
        headerClassName: 'p-4 md:p-16',
      },
      {
        id: 'action',
        width: 128,
        accessor: ({ row }) => (
          <div className="flex items-center">
            <IconButton
              onClick={(e) => {
                e.stopPropagation();
                deleteRole(row.id);
              }}
            >
              <DeleteIcon color="error" />
            </IconButton>
          </div>
        ),
      },
    ],
    [t, deleteRole, deleteRoles]
  );

  const {
    page,
    sortBy,
    search,
    perPage,
    sortDir,
    handleSearch,
    appliedFilters,
    ...tableInstance
  } = useDataTable({
    columns,
    initialPage: queryPage,
    data: resolvedData?.roles,
    initialSortBy: querySortBy,
    initialSearch: querySearch,
    initialPerPage: queryPerPage,
    initialSortDir: querySortDir,
  });

  // // Listen for changes in pagination and use the state to fetch our new data
  useEffect(() => {
    setQueryPage(page);
  }, [page]);

  useEffect(() => {
    setQueryPerPage(perPage);
  }, [perPage]);

  useEffect(() => {
    setQuerySearch(search);
  }, [search]);

  useEffect(() => {
    if (sortBy !== querySortBy) {
      setQuerySortBy(sortBy);
    }

    if (sortDir !== querySortDir) {
      setQuerySortDir(sortDir);
    }
  }, [sortBy, sortDir, querySortBy, querySortDir]);

  const handleOpenAddDialog = () => {
    setIsDialogOpen(true);
    setIsEditDialog(false);
  };

  const handleOpenEditDialog = (role) => {
    setIsDialogOpen(true);
    setIsEditDialog(true);
    setSelectedRole(role);
  };

  const handleCloseDialog = () => {
    setIsDialogOpen(false);
    setIsEditDialog(false);
    setSelectedRole(null);
  };

  return (
    <FusePageCarded
      classes={{
        content: 'flex',
        contentCard: 'overflow-hidden',
        header: 'min-h-36 h-36 sm:h-72 sm:min-h-72',
      }}
      header={
        <Header 
          onSearch={handleSearch}
          initialSearchValue={search}
          onOpenAddDialog={handleOpenAddDialog}
        />
      }
      content={
        <>
          <DataTable
            hover
            page={page}
            stickyHeader
            sortBy={sortBy}
            sortDir={sortDir}
            perPage={perPage}
            count={resolvedData?.count}
            onRowClick={handleOpenEditDialog}
            {...tableInstance}
          />
          {isDialogOpen && routes && (
            <RoleFormDialog
              routes={routes}
              edit={isEditDialog}
              isOpen={isDialogOpen}
              onDelete={deleteRole}
              onClose={handleCloseDialog}
              selectedRole={selectedRole}
            />
          )}
        </>
      }
      innerScroll
    />
  );
}

export default RolesManagement;
