import React, { useEffect, useMemo, useState } from 'react';
import {
  useTable,
  useSortBy,
  useRowSelect,
  usePagination,
  useGlobalFilter,
} from 'react-table';
import Fab from '@material-ui/core/Fab';
import Table from 'app/components/Table';
import { useTranslation } from 'react-i18next';
import { makeStyles } from '@material-ui/core/styles';
import FusePageCarded from '@fuse/core/FusePageCarded';
import UploadIcon from '@material-ui/icons/CloudUpload';
import TableCheckbox from 'app/components/TableCheckbox';
import ProductsMultiSelectMenu from '../../e-commerce/components/ProductsMultiSelectMenu';
import UserSettingsHeader from './UserSettingsHeader';
import useUserSettings from '../hooks/useUserSettings';
import Dialog from '../Dialog';
import UserDatas from './UserDatas';
const useStyles = makeStyles((theme) => ({
  fab: {
    position: 'absolute',
    bottom: theme.spacing(2),
    right: theme.spacing(2),
  },
}));

function UserSettings() {
  const classes = useStyles();
  const { t } = useTranslation('settings');
  const [openDialog, setopenDialog] = useState(false);
  const [DetailItems, setDetailItems] = useState();
  const handleClickOpen = (id) => {
    setopenDialog(true);
    setDetailItems(id.original);
  };
  const handleClose = () => {
    setopenDialog(false);
  };

  const columns = useMemo(
    () => [
      {
        Header: ({ selectedFlatRows }) => {
          const selectedRowIds = selectedFlatRows.map((row) => row.avatar);
          return (
            selectedFlatRows.length > 0 && (
              <ProductsMultiSelectMenu selectedIds={selectedRowIds} />
            )
          );
        },
        accessor: 'images',
        Cell: ({ row }) => (
          <img
            className="rounded"
            alt={row.displayName}
            src={row.avatar ? row.avatar : null}
          />
        ),
        className: 'w-52 px-4 md:px-0',
        width: 64,
        sortable: false,
      },
      { Header: t('USERNAME'), accessor: 'displayName', sortable: true },
      { Header: t('ROLE'), accessor: 'role', sortable: true },
      { Header: t('EMAIL'), accessor: 'email', sortable: true },
      { Header: t('NUMBER'), accessor: 'mobile', sortable: true },
    ],
    [t]
  );
  const [queryPageSize, setQueryPageSize] = useState(10);
  const [queryPageIndex, setQueryPageIndex] = useState(0);
  const [queryFilters, setQueryFilters] = useState({ search: '' });
  const [querySortBy, setQuerySortBy] = useState();
  const [querySortDesc, setQuerySortDesc] = useState();

  const {
    status,
    isLoading,
    resolvedData = { count: 0, users: [], pageCount: 0 },
    isFetching,
  } = useUserSettings({
    page: queryPageIndex,
    perPage: queryPageSize,
    search: queryFilters.search,
    sortBy: querySortBy,
    sortDesc: querySortDesc,
  });
  const tableInstance = useTable(
    {
      columns,
      manualSortBy: true,
      autoResetPage: false,
      manualPagination: true,
      disableMultiSort: true,
      manualGlobalFilter: true,
      data: resolvedData?.users,
      pageCount: 10,
      initialState: {
        pageIndex: queryPageIndex,
        pageSize: queryPageSize,
        globalFilter: queryFilters,
      },
    },
    useGlobalFilter,
    useSortBy,
    usePagination,
    useRowSelect,
    (hooks) => {
      hooks.allColumns.push((_columns) => [
        // Let's make a column for selection
        {
          id: 'selection',
          sortable: false,
          // The header can use the table's getToggleAllRowsSelectedProps method
          // to render a checkbox.  Pagination is a problem since this will select all
          // rows even though not all rows are on the current page.  The solution should
          // be server side pagination.  For one, the clients should not download all
          // rows in most cases.  The client should only download data for the current page.
          // In that case, getToggleAllRowsSelectedProps works fine.
          Header: ({ getToggleAllRowsSelectedProps }) => (
            <div>
              <TableCheckbox {...getToggleAllRowsSelectedProps()} />
            </div>
          ),
          // The cell can use the individual row's getToggleRowSelectedProps method
          // to the render a checkbox
          Cell: ({ row }) => (
            <div>
              <TableCheckbox
                {...row.getToggleRowSelectedProps()}
                onClick={(ev) => ev.stopPropagation()}
              />
            </div>
          ),
        },
        ..._columns,
      ]);
    }
  );

  const {
    setGlobalFilter,
    state: { pageIndex, pageSize, globalFilter, sortBy },
  } = tableInstance;

  // Listen for changes in pagination and use the state to fetch our new data
  useEffect(() => {
    setQueryPageIndex(pageIndex);
  }, [pageIndex]);

  useEffect(() => {
    setQueryPageSize(pageSize);
  }, [pageSize]);

  useEffect(() => {
    setQueryFilters(globalFilter);
  }, [globalFilter]);

  useEffect(() => {
    if (sortBy?.[0]?.id !== querySortBy) {
      setQuerySortBy(sortBy?.[0]?.id);
    }

    if (sortBy?.[0]?.desc !== querySortDesc) {
      setQuerySortDesc(sortBy?.[0]?.desc);
    }
  }, [sortBy, querySortBy, querySortDesc]);
  return (
    <FusePageCarded
      classes={{
        content: 'flex',
        contentCard: 'overflow-hidden',
        header: 'min-h-72 h-72 sm:h-136 sm:min-h-136',
      }}
      header={<UserSettingsHeader />}
      content={
        <>
          <Table
            clicked={handleClickOpen}
            showFilters={false}
            isLoading={isLoading}
            count={10}
            tableInstance={tableInstance}
            isFetching={status === 'success' && isFetching}
          />
          <Dialog dialog={openDialog} closeDialog={handleClose}>
            <UserDatas
              id={DetailItems?.id}
              address={DetailItems?.address}
              displayName={DetailItems?.displayName}
              email={DetailItems?.email}
              mobile={DetailItems?.mobile}
              role={DetailItems?.role}
              status={DetailItems?.status}
              Nationalid={DetailItems?.Nationalid}
              postal={DetailItems?.postal}
            />
          </Dialog>
          <Fab
            color="primary"
            className={classes.fab}
            aria-label="upload excel file"
          >
            <UploadIcon />
          </Fab>
        </>
      }
      innerScroll
    />
  );
}

export default UserSettings;
